#include "scriptinterface.h"
#include "findclosestmapedge.h"

#include "Fed_6MissionText.h"
#include "Fed_6MissionMaps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 ImperialDestroyed = 0;
int32 PlayerDestroyed = 0;
int32 DeckerDestroyed = 0; // counts if decker is captured or destroyed
int32 DeckerCaptured = 0; // counts if decker is captured
int32 DeckerBlownUp = 0; // flag ONLY for Decker's ship blowing up

bool AtWarWithNeutral = false;
bool AtWarWithImperial = false;
bool ImperialPatrollingBase = false;
bool ObjectiveMessageDisplayed = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

const float kDamageThreshold = 70.0;
const float kPatrolAttackRadius = 30.0;
const int32 kBaseX = 20;
const int32 kBaseY = 20;
const int32 kMapEdgeX = 40;
const int32 kMapEdgeY = 40;


void KlingonCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship);
void RomulanCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship);
void GornCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship);
void NeutralPatrol(tTeamInfo* PatrolTeam);
void ImperialPatrol(tTeamInfo* PatrolTeam);
void CheckWinCondition( tScript& tScript );

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kImperialTeam,
	kNeutralTeam,
	kDeckerTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kFedMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	//info.fEnemyRace = info.fPlayerRace;
	info.fEnemyRace = (fMission.mRandomInt32 (1, 100) < 33) ? kKlingon
					  : (fMission.mRandomInt32 (1, 100) < 66) ? kRomulan : kGorn;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kImperialTeam, kMirror);
	fMission.mSetTeamRace (kDeckerTeam, kMirror);

	// LNF: The Neutral team is the worst relations to Fed.
	fMission.mSetTeamRace (kNeutralTeam, info.fEnemyRace);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kImperialTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kNeutralTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kDeckerTeam, kTeamHostile, kGoalCapture, 1.0);
	
	fMission.mSetTeamRelations (kImperialTeam, kNeutralTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kImperialTeam, kDeckerTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kNeutralTeam, kDeckerTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kDeckerTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
	
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* ImperialTeam = fMission.mGetTeamHandle (kImperialTeam);
	tTeamInfo* NeutralTeam = fMission.mGetTeamHandle (kNeutralTeam);
	tTeamInfo* DeckerTeam = fMission.mGetTeamHandle (kDeckerTeam);

	tShipInfo* Ship = NULL;

	// LNF probably don't need this stuff for this mission
	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	// create Imperial team
	if ( NumberOfPlayersShips == 1 )
	{
		ImperialTeam->mCreateShip( AveragePlayerBPV, kClassFrigate, kClassFrigate, 80, kStartPosition_V );
	}
	else if ( NumberOfPlayersShips == 2 )
	{
		ImperialTeam->mCreateShip( AveragePlayerBPV, kClassFrigate, kClassLightCruiser, 80, kStartPosition_V );
	}
	else if ( NumberOfPlayersShips > 2 )
	{
		ImperialTeam->mCreateFleet (2, (2 * AveragePlayerBPV), kClassFrigate, kClassLightCruiser, 80, kStartPosition_V);
	}

	// create Neutral team
	NeutralTeam->mCreateFleet (4, (4 * (TotalBPV/NumberOfPlayersShips)), kClassFrigate, kClassLightCruiser, 70, kStartPosition_P);
	NeutralTeam->mCreateShip (kMaxBPV, kClassBaseStation, kClassBaseStation, 100, kStartPosition_O);

	// create Decker's ship
	DeckerTeam->mCreateShip ( AveragePlayerBPV, kClassHeavyCruiser, kClassHeavyCruiser, 100, kStartPosition_U);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* ImperialTeam = fMission.mGetTeamHandle (kImperialTeam);
	tTeamInfo* NeutralTeam = fMission.mGetTeamHandle (kNeutralTeam);
	tTeamInfo* DeckerTeam = fMission.mGetTeamHandle (kDeckerTeam);
	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
	}

	// configure Imperial ships
	for (Ship = ImperialTeam->mGetFirstShip(); Ship != NULL; Ship = ImperialTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		Ship->mDefendEntity( DeckerTeam->mGetFirstShip(), 20.0, 20.0, kVeryHighPriority );
	}
	// make Imperial patrol
	//ImperialPatrol( ImperialTeam );

	// configure Neutral ships
	for (Ship = NeutralTeam->mGetFirstShip(); Ship != NULL; Ship = NeutralTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents( kNotifyDamage );
	}
	// make Neutrals patrol
	NeutralPatrol(NeutralTeam);

	// configure Decker's ships
	for (Ship = DeckerTeam->mGetFirstShip(); Ship != NULL; Ship = DeckerTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		Ship->mChangeName( kWillDecker_name );
	}
	// make Decker patrol
	ImperialPatrol( DeckerTeam );
	
	//fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 2);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* ImperialTeam = fMission.mGetTeamHandle (kImperialTeam);
	tTeamInfo* NeutralTeam = fMission.mGetTeamHandle (kNeutralTeam);
	tTeamInfo* DeckerTeam = fMission.mGetTeamHandle (kDeckerTeam);

	switch (Event)
	{
		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);

				if ( HostileShip != NULL )
				{
					if ( HostileShip->mGetTeam() == kPlayerTeam )
					{
						if ((( Ship->mGetTeam() == kDeckerTeam ) ||
							 ( Ship->mGetTeam() == kImperialTeam )) &&
							 ( AtWarWithImperial == false ))
						{
							fMission.mGetTeamHandle(HostileShip->mGetTeam())->mClearCommands();
							fMission.mSetTeamRelations (kPlayerTeam, kImperialTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							fMission.mSetTeamRelations (kPlayerTeam, kDeckerTeam, kTeamAtWar, kGoalCapture, 1.0); //***
							fMission.mSetTeamRelations (kDeckerTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0); //***
							AtWarWithImperial = true;
						}
						else if (( Ship->mGetTeam() == kNeutralTeam ) && ( AtWarWithNeutral == false ))
						{
							AtWarWithNeutral = true;
							NeutralTeam->mClearCommands();
							fMission.mSetTeamRelations (kPlayerTeam, kNeutralTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						}
					}
					else
					{
						Ship->mSetShipCallbackEvents( kNotifyDamage );
					}
				}
			}
			break;

		case kNotifyAtDistance:
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// setup team relations and target goals
					fMission.mSetTeamRelations (kPlayerTeam, kImperialTeam, kTeamAtWar, kGoalDestroy, 1.0); //***
					fMission.mSetTeamRelations (kPlayerTeam, kNeutralTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kDeckerTeam, kTeamAtWar, kGoalCapture, 1.0);//***
					
					fMission.mSetTeamRelations (kImperialTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0); //***
					fMission.mSetTeamRelations (kImperialTeam, kNeutralTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kImperialTeam, kDeckerTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kNeutralTeam, kDeckerTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kDeckerTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);//***

					fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );
					break;

				case kTimerTurnsSet_8:
					// set the Neutral Team comm buttons back to HAIL
					for (Ship = NeutralTeam->mGetFirstShip(); Ship != NULL; Ship = NeutralTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set the Imperial Team comm buttons back to HAIL
					for (Ship = ImperialTeam->mGetFirstShip(); Ship != NULL; Ship = ImperialTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Decker Team comm buttons back to HAIL
					for (Ship = DeckerTeam->mGetFirstShip(); Ship != NULL; Ship = DeckerTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;

		
		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kImperialTeam)
			{
				ImperialDestroyed++;
				if ((ImperialTeam->mGetShipsInPlayCount() == 0) &&
					(DeckerTeam->mGetShipsInPlayCount() == 0))
				{
					CheckWinCondition( *this );
				}
			}
			else if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			else if (Ship->mGetTeam() == kDeckerTeam)
			{
				DeckerDestroyed++;

				DeckerBlownUp = 1;

				if ((DeckerTeam->mGetShipsInPlayCount() == 0) &&
					(ImperialTeam->mGetShipsInPlayCount() == 0))
				{
					CheckWinCondition( *this );
				}
			}
			break;

		case kNotifyOnCapture:
			{
				fMission.mAddToBonusPrestige( kCaptureBonus );
				// LNF should I have some kind of capture counter or something?

				if ( Ship->mGetTeam() == kDeckerTeam )
				{
					DeckerCaptured = 1;
					DeckerDestroyed = 1;
					CheckWinCondition( *this );
				}
			}
			break;

		case kNotifyDisengaged:
			if ( PlayerTeam->mGetShipsInPlayCount() == 0 )
			{
				fMission.mSetGoalIndicator( kFalse, 0, 0 );
			}
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kImperialTeam:
								fMission.mDisplayMessage (kImpOK_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kNeutralTeam:
								switch (NeutralTeam->mGetRace())
								{
									case kKlingon:
										KlingonCommMessages(*this, fEventID, Ship);
										break;
									case kRomulan:
										RomulanCommMessages(*this, fEventID, Ship);
										break;
									case kGorn:
										GornCommMessages(*this, fEventID, Ship);
										break;
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kDeckerTeam:
								fMission.mDisplayMessage (kDeckerHail_msg, kMissionScriptMessage);
								//fMission.mDisplayMessage (kDeckerRetreating_msg, kMissionScriptMessage);
								//fMission.mDisplayMessage (kDeckerDisabled_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kImperialTeam:
								fMission.mDisplayMessage (kImpOK_msg, kMissionScriptMessage);
								//fMission.mDisplayMessage (kImpDamaged_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kNeutralTeam:
								switch (NeutralTeam->mGetRace())
								{
									case kKlingon:
										KlingonCommMessages(*this, fEventID, Ship);
										break;
									case kRomulan:
										RomulanCommMessages(*this, fEventID, Ship);
										break;
									case kGorn:
										GornCommMessages(*this, fEventID, Ship);
										break;
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kDeckerTeam:
								fMission.mDisplayMessage (kDeckerSurrenderOK_msg, kMissionScriptMessage);
								//fMission.mDisplayMessage (kDeckerSurrenderDamaged_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kImperialTeam:
								fMission.mDisplayMessage (kImpOK_msg, kMissionScriptMessage);
								//fMission.mDisplayMessage (kImpDamaged_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kNeutralTeam:
								switch (NeutralTeam->mGetRace())
								{
									case kKlingon:
										KlingonCommMessages(*this, fEventID, Ship);
										break;
									case kRomulan:
										RomulanCommMessages(*this, fEventID, Ship);
										break;
									case kGorn:
										GornCommMessages(*this, fEventID, Ship);
										break;
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;

							case kDeckerTeam:
								fMission.mDisplayMessage (fMission.mRandomInt32 (kDeckerTaunt1_msg, kDeckerTaunt4_msg), kMissionScriptMessage);
								//fMission.mDisplayMessage (fMission.mRandomInt32 (kDeckerDamagedTaunt1_msg, kDeckerDamagedTaunt4_msg), kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;
						}
						break;
				}
			}
			break;

	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	// LNF check for decker captured?
	VictoryPoints = (DeckerCaptured * 70) + (DeckerDestroyed * 60) + (ImperialDestroyed * 15);
	
	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if ((VictoryPoints <= 49) && (VictoryPoints >= 35))
	{
		MyVictoryLevel = kDefeat;
	}
	else if (VictoryPoints <= 34)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

		if (VictoryLevel == kAstoundingVictory)
		{	
			fMission.mAddToDebriefingList (kBlankLine_msg);
			fMission.mAddToDebriefingList (kBlankLine_msg);
			fMission.mAddToDebriefingList (kVictory2_msg);
		}
		else if (VictoryLevel == kVictory)
		{
			fMission.mAddToDebriefingList (kBlankLine_msg);
			fMission.mAddToDebriefingList (kVictory2_msg);
		}

		if (VictoryLevel < kDraw)
		{
			fMission.mAddToDebriefingList (kBlankLine_msg);
			fMission.mAddToDebriefingList (kDismissedLong_msg);
		}
		else
		{
			fMission.mAddToDebriefingList (kBlankLine_msg);
			fMission.mAddToDebriefingList (kDismissedShort_msg);
		}
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}

	// LNF try this out: adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	if ( DeckerCaptured == 1 )
	{
		fMission.mAddToDebriefingList( kDeckerCapturedTrue_msg );
	}
	else if ( DeckerDestroyed == 1 )
	{
		fMission.mAddToDebriefingList( kDeckerDestroyedTrue_msg );
	}
	else
	{
		SuccessCount++;
	}

//	( DeckerDestroyed != 0 ) ? fMission.mAddToDebriefingList( kDeckerDestroyedTrue_msg ) : SuccessCount++;
	( ImperialDestroyed != 0 ) ? fMission.mAddToDebriefingList( kImperialDestroyedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	if (( DeckerCaptured == 0 ) && ( DeckerDestroyed == 0 ))
	{
		fMission.mAddToDebriefingList( kDeckerCapturedFalse_msg );
	}
	else
	{
		FailureCount++;
	}

//	( DeckerDestroyed == 0 ) ? fMission.mAddToDebriefingList( kDeckerDestroyedFalse_msg ) : FailureCount++;
	( ImperialDestroyed == 0 ) ? fMission.mAddToDebriefingList( kImperialDestroyedFalse_msg ) : FailureCount++;
	if ( FailureCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// Display comm messages for Klingon
void KlingonCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship)
{
	tTeamInfo* PlayerTeam = tScript.fMission.mGetTeamHandle (tScript.kPlayerTeam);
	tTeamInfo* ImperialTeam = tScript.fMission.mGetTeamHandle (kImperialTeam);
	tTeamInfo* NeutralTeam = tScript.fMission.mGetTeamHandle (kNeutralTeam);
	tTeamInfo* DeckerTeam = tScript.fMission.mGetTeamHandle (kDeckerTeam);

 	
	if ( fEventID == kButtonHail_text )
	{
		if ( AtWarWithNeutral == false )
		{
			tScript.fMission.mDisplayMessage( kKliAllNE_msg, kMissionScriptMessage );
		}
		else
		{
			tScript.fMission.mDisplayMessage( kKliHailCombat_msg, kMissionScriptMessage );
		}
	}
	else if ( fEventID == kButtonSurrender_text )
	{
		if ( AtWarWithNeutral == false )
		{
			tScript.fMission.mDisplayMessage( kKliAllNE_msg, kMissionScriptMessage);
		}
		else
		{
			if ( Ship->mGetDamagePercent() <= kDamageThreshold )
			{
				tScript.fMission.mDisplayMessage( kKliSurrenderOK_msg, kMissionScriptMessage);
			}
			else
			{
				tScript.fMission.mDisplayMessage(kKliSurrenderDamaged_msg, kMissionScriptMessage);
			}
		}
	}
	else if ( fEventID == kButtonTaunt_text )
	{
		if ( AtWarWithNeutral == false )
		{
			tScript.fMission.mDisplayMessage( kKliAllNE_msg, kMissionScriptMessage );
		}
		else
		{
			tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kKliTaunt_1_msg, kKliTaunt_4_msg), kMissionScriptMessage);
		}
	}
}

//---------------------------------------------------------------------------------------------
// Display comm messages for Romulan
void RomulanCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship)
{
	tTeamInfo* PlayerTeam = tScript.fMission.mGetTeamHandle (tScript.kPlayerTeam);
	tTeamInfo* ImperialTeam = tScript.fMission.mGetTeamHandle (kImperialTeam);
	tTeamInfo* NeutralTeam = tScript.fMission.mGetTeamHandle (kNeutralTeam);
	tTeamInfo* DeckerTeam = tScript.fMission.mGetTeamHandle (kDeckerTeam);


	if ( fEventID == kButtonHail_text )
	{
		if ( AtWarWithNeutral == false )
		{
			tScript.fMission.mDisplayMessage( kRomAllNE_msg, kMissionScriptMessage );
		}
		else
		{
			tScript.fMission.mDisplayMessage( kRomHailCombat_msg, kMissionScriptMessage );
		}
	}
	else if ( fEventID == kButtonSurrender_text )
	{
		if ( AtWarWithNeutral == false )
		{
			tScript.fMission.mDisplayMessage( kRomAllNE_msg, kMissionScriptMessage );
		}
		else
		{
			tScript.fMission.mDisplayMessage( kRomHailCombat_msg, kMissionScriptMessage );
		}
	}
	else if ( fEventID == kButtonTaunt_text )
	{
		if ( AtWarWithNeutral == false )
		{
			tScript.fMission.mDisplayMessage( kRomAllNE_msg, kMissionScriptMessage );
		}
		else
		{
			tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kRomTaunt_1_msg, kRomTaunt_4_msg), kMissionScriptMessage);
		}
	}
}

//---------------------------------------------------------------------------------------------
// Display comm messages for Gorn
void GornCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship)
{
	tTeamInfo* PlayerTeam = tScript.fMission.mGetTeamHandle (tScript.kPlayerTeam);
	tTeamInfo* ImperialTeam = tScript.fMission.mGetTeamHandle (kImperialTeam);
	tTeamInfo* NeutralTeam = tScript.fMission.mGetTeamHandle (kNeutralTeam);
	tTeamInfo* DeckerTeam = tScript.fMission.mGetTeamHandle (kDeckerTeam);

	if ( fEventID == kButtonHail_text )
	{
		if ( AtWarWithNeutral == false )
		{
			tScript.fMission.mDisplayMessage( kGornAllNE_msg, kMissionScriptMessage );
		}
		else
		{
			tScript.fMission.mDisplayMessage( kGornHailCombat_msg, kMissionScriptMessage );
		}
	}
	else if ( fEventID == kButtonSurrender_text )
	{
		if ( AtWarWithNeutral == false )
		{
			tScript.fMission.mDisplayMessage( kGornAllNE_msg, kMissionScriptMessage );
		}
		else
		{
			if ( Ship->mGetDamagePercent() <= kDamageThreshold )
			{
				tScript.fMission.mDisplayMessage( kGornSurrenderOK_msg, kMissionScriptMessage);
			}
			else
			{
				tScript.fMission.mDisplayMessage(kGornSurrenderDamaged_msg, kMissionScriptMessage);
			}
		}
	}
	else if ( fEventID == kButtonTaunt_text )
	{
		if ( AtWarWithNeutral == false )
		{
			tScript.fMission.mDisplayMessage( kGornAllNE_msg, kMissionScriptMessage );
		}
		else
		{
			tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kGornTaunt_1_msg, kGornTaunt_4_msg), kMissionScriptMessage);
		}
	}
}

//---------------------------------------------------------------------------------------------
// Neutral Patrol routes
void NeutralPatrol(tTeamInfo* PatrolTeam)
{
	//tShipInfo* Ship;

	tMapPoint waypoints_1[6]= // ship P
	{
		16, 22,
		18, 25,
		24, 25,
		26, 20,
		23, 16,
		18, 16
	};

	tMapPoint waypoints_2[6]= // ship Q
	{
		4, 27,
		5, 22,
		9, 19,
		15, 22,
		14, 28,
		9, 30
	};

	tMapPoint waypoints_3[6]= // ship R
	{
		27, 27,
		24, 26,
		22, 23,
		24, 19,
		28, 18,
		31, 22
	};

	tMapPoint waypoints_4[6]= // ship S
	{
		27, 10,
		22, 10,
		19, 13,
		24, 18,
		29, 16,
		30, 12
	};

	tMapPoint PatrolPoints[4][10];

	PatrolPoints[0][0] = waypoints_1[0];
	PatrolPoints[0][1] = waypoints_1[1];
	PatrolPoints[0][2] = waypoints_1[2];
	PatrolPoints[0][3] = waypoints_1[3];
	PatrolPoints[0][4] = waypoints_1[4];
	PatrolPoints[0][5] = waypoints_1[5];
	PatrolTeam->mGetThisShip(1)->mPatrol( kPatrolAttackRadius, 6, &PatrolPoints[0][0], kHighPriority );

	PatrolPoints[1][0] = waypoints_2[0];
	PatrolPoints[1][1] = waypoints_2[1];
	PatrolPoints[1][2] = waypoints_2[2];
	PatrolPoints[1][3] = waypoints_2[3];
	PatrolPoints[1][4] = waypoints_2[4];
	PatrolPoints[1][5] = waypoints_2[5];
	PatrolTeam->mGetThisShip(2)->mPatrol( kPatrolAttackRadius, 6, &PatrolPoints[1][0], kHighPriority );

	PatrolPoints[2][0] = waypoints_3[0];
	PatrolPoints[2][1] = waypoints_3[1];
	PatrolPoints[2][2] = waypoints_3[2];
	PatrolPoints[2][3] = waypoints_3[3];
	PatrolPoints[2][4] = waypoints_3[4];
	PatrolPoints[2][5] = waypoints_3[5];
	PatrolTeam->mGetThisShip(3)->mPatrol( kPatrolAttackRadius, 6, &PatrolPoints[2][0], kHighPriority );

	PatrolPoints[3][0] = waypoints_4[0];
	PatrolPoints[3][1] = waypoints_4[1];
	PatrolPoints[3][2] = waypoints_4[2];
	PatrolPoints[3][3] = waypoints_4[3];
	PatrolPoints[3][4] = waypoints_4[4];
	PatrolPoints[3][5] = waypoints_4[5];
	PatrolTeam->mGetThisShip(4)->mPatrol( kPatrolAttackRadius, 6, &PatrolPoints[3][0], kHighPriority );
}

//---------------------------------------------------------------------------------------------
// Imperial Patrol routes
void ImperialPatrol(tTeamInfo* PatrolTeam)
{
	tShipInfo* Ship = NULL;

	tMapPoint waypoints_1[6]=
	{
		19, 18,
		18, 20,
		19, 22,
		21, 22,
		22, 20,
		21, 18
	};

	tMapPoint PatrolPoints[4][10];

	PatrolPoints[0][0] = waypoints_1[0];
	PatrolPoints[0][1] = waypoints_1[1];
	PatrolPoints[0][2] = waypoints_1[2];
	PatrolPoints[0][3] = waypoints_1[3];
	PatrolPoints[0][4] = waypoints_1[4];
	PatrolPoints[0][5] = waypoints_1[5];

	for (Ship = PatrolTeam->mGetFirstShip(); Ship != NULL; Ship = PatrolTeam->mGetNextShip())
	{
		Ship->mPatrol( kPatrolAttackRadius, 6, &PatrolPoints[0][0], kHighPriority );
	}
}

//---------------------------------------------------------------------------------------------
// win condition
void CheckWinCondition( tScript& tScript )
{
	if ((( DeckerCaptured == 1 ) || ( DeckerDestroyed == 1 ))
		&& ( ObjectiveMessageDisplayed == false ))
	{
		ObjectiveMessageDisplayed = true;
		tScript.fMission.mDisplayMessage( kPrimaryObjectivesComplete_msg, kMissionScriptMessage );
		FindClosestMapEdge( tScript, tScript.fPlayerShip, kMapEdgeX, kMapEdgeY );
	}
}

//---------------------------------------------------------------------------------------------
// randomize the ship class for the computer ships
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass )
{
/*
	kClassShuttle = 0,				// smallest hull size
	kClassFreighter,
	kClassFrigate,
	kClassDestroyer,
	kClassWarDestroyer,
	kClassLightCruiser,
	kClassNewHeavyCruiser,
	kClassHeavyCruiser,
	kClassHeavyBattlecruiser,
	kClassDreadnought,
	kClassBattleship,				// largest hull size
*/
	int32 RandomNum = tScript.fMission.mRandomInt32(1, 100);
	
	// 70% use whatever the difficulty level sets
	// 15% use a class size smaller
	// 10% use 2 class sizes smaller
	// 5% use a class size larger
	if ( RandomNum > 70 )
	{
		return LargestEnemyShipClass;
	}
	else if (( RandomNum < 30 ) && ( RandomNum >= 15 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 1 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else if (( RandomNum < 15 ) && ( RandomNum >= 5 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass - 2 );

		if ( LargestEnemyShipClass > kClassFrigate )
		{
			return LargestEnemyShipClass;
		}
		else
		{
			return kClassFrigate;
		}
	}
	else // if (( RandomNum < 5 ) && ( RandomNum >= 1 ))
	{
		LargestEnemyShipClass = (eClassTypes) ( (int32)LargestEnemyShipClass + 1 );

		if ( LargestEnemyShipClass > kClassBattleship )
		{
			return kClassBattleship;
		}
		else
		{
			return LargestEnemyShipClass;
		}
	}
}
