#include "scriptinterface.h"

#include "Gen_MonsterText.h"
#include "Gen_MonsterMaps.h"

int32 MapNumber = 0;
int32 RandomMonster = 0;
int32 DiffLevel = 0;
int32 BonusBalancePoints = 0;
float PlayerHealth = 0.0;
float EnemyHealth = 0.0;
bool NebulaMap = false;
bool AsteroidMap = false;
bool MonsterScanned = false;
bool PlayerDisengaged = false;

enum eMonsterType
{
	kSpaceshell = 0,
	kMeater,
	kSunglider,
	kAstrominer,
	kPlanet,
};

enum eDiffLevel
{
	kDiffEasy = 0,
	kDiffMedium,
	kDiffHard,
};

const tString MonsterName[5][3] =
{
	{ "M-SSS", "M-SSM", "M-SSL" },
	{ "M-MTS", "M-MTM", "M-MTL" },
	{ "M-SGS", "M-SGM", "M-SGL" },
	{ "M-AMS", "M-AMM", "M-AML" },
	{ "PL1", "PL3", "PL0" },
};

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam, kPlanetTeam,
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kEnemyTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kKlingMission | kRomulanMission | kGornMission | kLyranMission;
	Specs->fEnemyRace = kNonPirateMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 3;
	Specs->fMaxDate = 50;
	Specs->fWait = 6;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
 	fMission.mSetPrestigePoints( 100, 75, 50, 25, 0 );

	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, kMonster );
	fMission.mSetTeamRace( kPlanetTeam, info.fPlayerRace );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kEnemyTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

	// pick a random map
	MapNumber = fMission.mRandomInt32( 1, TotalMaps );

	if( MapNumber == 3 )
	{
		NebulaMap = true;
	}

	if( MapNumber > 1 )
	{
		AsteroidMap = true;
	}

	return( MapNumber );
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle( kPlanetTeam );

	int32 PlayerBPV = PlayerTeam->mGetCombatBPV();
	int32 MonsterBPV = 0;
	int32 MonsterCount = 0;
	int32 Position = fMission.mRandomInt32( kStartPosition_M, kStartPosition_W );

	RandomMonster = fMission.mRandomInt32( kSpaceshell, kAstrominer );

	if(( RandomMonster == kAstrominer ) && ( AsteroidMap == false ))
	{
		RandomMonster = kSunglider;
	}

	if(( RandomMonster == kSunglider ) && ( NebulaMap == true ))
	{
		RandomMonster = kSpaceshell;
	}

	if( DiffLevel == kDiffEasy )
	{
		PlayerBPV = PlayerBPV * .75;
	}

	if( DiffLevel == kDiffHard )
	{
		PlayerBPV = PlayerBPV * 1.5;
	}

	tString CurrMonster = MonsterName[ RandomMonster ][ DiffLevel ];
	EnemyTeam->mCreateShip( CurrMonster.mCharStr(), Position );
	MonsterBPV = EnemyTeam->mGetCombatBPV();

	while(( PlayerBPV > MonsterBPV ) && ( MonsterCount < 12 ))
	{
		Position++;
		MonsterCount++;

		if( Position > kStartPosition_Z )
		{
			Position = kStartPosition_M;
		}

		tString CurrMonster = MonsterName[ RandomMonster ][ kDiffEasy ];
		EnemyTeam->mCreateShip( CurrMonster.mCharStr(), Position );
		MonsterBPV = EnemyTeam->mGetCombatBPV();
	}

	CurrMonster = MonsterName[ kPlanet ][ MapNumber - 1 ];
	PlanetTeam->mCreateShip( CurrMonster.mCharStr(), kStartPosition_Specified, 20, 20 );
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	int32 MessageNumber = kBriefingA_msg;

	if( AsteroidMap == true )
	{
		MessageNumber = kBriefingB_msg;
	}

	if( NebulaMap == true )
	{
		MessageNumber = kBriefingC_msg;
	}

	return( MessageNumber );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle( kPlanetTeam );
	tShipInfo* Ship;

	int32 PBPV = PlayerTeam->mGetCombatBPV();
	int32 EBPV = EnemyTeam->mGetCombatBPV();
	BonusBalancePoints = ( 20 * EBPV / PBPV ) - 20;

	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for( Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip() )
	{
		Ship->mSetShipThrust( 5 );
	}

	// configure the planet
	Ship = PlanetTeam->mGetFirstShip();
	Ship->mChangeName( kGojira_name );
	Ship->mSetShipAIControlled( kFalse );

}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tShipInfo* Monster;

	PlayerHealth = 100.0f - PlayerTeam->mGetTotalDamagePercent();
	EnemyHealth = 100.0f - EnemyTeam->mGetTotalDamagePercent();

	int32 Enemies = EnemyTeam->mGetShipsInPlayCount();

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			fMission.mSetTeamRelations( kPlayerTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
			fMission.mSetTeamRelations( kEnemyTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
			break;

		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerDestroyed;
				}
			}
			
			if( Enemies == 0 )
			{
				fMission.mAddToBonusPrestige( BonusBalancePoints );
				return kPlayerWins;
			}
			break;

		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				fMission.mAddToBonusPrestige( BonusBalancePoints );
				PlayerDisengaged = true;
				return kPlayerDisengaged;
			}
			break;

		case kNotifyDeepScan:
			if(( fIsDeepScanOn == kTrue ) && ( MonsterScanned == false ))
			{
				Ship = fPlayerShip;

				for( Monster = EnemyTeam->mGetFirstShip(); Monster != NULL; Monster = EnemyTeam->mGetNextShip() )
				{
					if(( Ship->mGetTacticalDistance( Monster ) < 10 ) && ( MonsterScanned == false ))
					{
						MonsterScanned = true;
						fMission.mDisplayMessage( kMonsterScanned_msg, kScienceScriptMessage, Monster );
					}
				}
			}
			break;

		case kNotifyProbeLaunch:
			Monster = fMission.mGetShipTarget( fPlayerShip );

			if( Monster != NULL )
			{
				if(( Monster->mGetTeam() == kEnemyTeam ) && ( MonsterScanned == false ))
				{
					MonsterScanned = true;
					fMission.mDisplayMessage( kMonsterProbed_msg, kScienceScriptMessage, Monster );
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	if(( PlayerDisengaged == true ) && ( MonsterScanned == false ))
	{
		if( PlayerHealth > EnemyHealth )
		{
			return kVictory;
		}
		else if( PlayerHealth < ( EnemyHealth / 2 ))
		{
			return kDevastatingDefeat;
		}
		else 
		{
			return kDefeat;
		}
	}

	if(( PlayerHealth > 90.0 ) && ( MonsterScanned == true ))
	{
		return kAstoundingVictory;
	}

	if( PlayerHealth < 25.0 ) 
	{
		return kDraw;
	}

	// By default
	return kVictory;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
