#include "scriptinterface.h"

#include "MissionText.h"
#include "MissionMaps.h"

int32 PlayerCaptured = 0;
int32 EnemyCaptured = 0;
float PlayerHealth = 0.0;
float EnemyHealth = 0.0;
iTruth PlayerDisengaged = kFalse;
int32 BonusBalancePoints = 0;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kEnemyTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kEnemyBorder | kWarzone | kEnemyCapital | kEnemyColonial;
	Specs->fPlayerRace = kNonPirateMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecOneShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 7;
	Specs->fMaxDate = 50;
	Specs->fWait = 3;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (150, 80, 40, 20, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, info.fEnemyRace);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0);
	fMission.mSetTeamRelations (kEnemyTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);

	// pick a random map
	int32 MapNumber = fMission.mRandomInt32 (1, TotalMaps);
	return (MapNumber);
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	Ship.fShipPosition = fMission.mRandomInt32 (kStartPosition_P, kStartPosition_R);
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	// create enemy ship
	int32 RandomPosition = fMission.mRandomInt32 (kStartPosition_X, kStartPosition_Z);

	// Get the total BPV of the player and modify by 80% to 130%
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .7;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.2;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	int32 DifLev = fMission.mGetDifficultyLevel();

	switch ( DifLev )
	{
	case 0:
		{
			TotalBPV *= 0.70f;
		}
		break;
	case 1:
		{
			// do nothing
		}
		break;
	case 2:
		{
			TotalBPV *= 1.5f;
		}
		break;
	}

	BonusBalancePoints = ( 20 * TotalBPV / PlayerTeam->mGetCombatBPV() ) - 20;

	// Any size ship within the appropriate BPV and a 33% chance of a variant
	EnemyTeam->mCreateShip(TotalBPV, kClassFrigate, kClassBattleship, 0, RandomPosition);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	int32 MessageNumber = fMission.mRandomInt32 (kBriefingA_msg, kBriefingC_msg);

	return (MessageNumber);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tShipInfo* Ship;

	int32 PBPV = PlayerTeam->mGetCombatBPV();
	int32 EBPV = EnemyTeam->mGetCombatBPV();
	BonusBalancePoints = ( 20 * EBPV / PBPV ) - 20;

	// configure player ship(s)
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mSetShipThrust( 5 );
	Ship->mSetAlertStatus( kAlertYellow );

	for (Ship = EnemyTeam->mGetFirstShip (); Ship != NULL; Ship = EnemyTeam->mGetNextShip ())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
	}

	fMission.mSetTimedInterval( kTimerLimitSet, 1 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	PlayerHealth = 100.0f - PlayerTeam->mGetTotalDamagePercent();
	EnemyHealth = 100.0f - EnemyTeam->mGetTotalDamagePercent();

	switch (Event)
	{
		case kNotifyAtTimedInterval:
			fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );
			break;

		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				PlayerDisengaged = kTrue;
				fMission.mAddToBonusPrestige( BonusBalancePoints );
				return kPlayerWins;
			}
		break;

		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kPlayerTeam)
			{
				return kPlayerDestroyed;
			}
			else
			{
				fMission.mAddToBonusPrestige( BonusBalancePoints );
				return kEnemyDestroyed;
			}
			break;

		case kNotifyOnCapture:
			if (Ship->mGetOriginalTeam() == kPlayerTeam)
			{
				PlayerCaptured++;
				// LNF if it does/will change the 1 to 0
				if( PlayerTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerCaptured;
				}
			}
			else
			{
				EnemyCaptured++;
				fMission.mAddToBonusPrestige( kCaptureBonus );
				fMission.mAddToBonusPrestige( BonusBalancePoints );
				return kPlayerWins;
			}
			break;

		case kNotifyCommButtonPress:
			fMission.mDisplayMessage (kStatic_msg, kMissionScriptMessage);
			break;

	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	if ( PlayerDisengaged )
	{
		if ( PlayerHealth > EnemyHealth )
		{
			fMission.mAdjustDynaverse( kAdjustWarProgression, +1, EnemyTeam->mGetRace() );
			return kVictory;
		}
		else if ( PlayerHealth < ( EnemyHealth / 2 ) )
		{
			fMission.mAdjustDynaverse( kAdjustWarProgression, -2, EnemyTeam->mGetRace() );
			return kDevastatingDefeat;
		}
		else 
		{
			fMission.mAdjustDynaverse( kAdjustWarProgression, -1, EnemyTeam->mGetRace() );
			return kDefeat;
		}
	}


	if ( ( PlayerHealth > 90.0 ) || ( EnemyCaptured ) )
	{
		fMission.mAdjustDynaverse( kAdjustWarProgression, +2, EnemyTeam->mGetRace() );
		return kAstoundingVictory;
	}


	if ( PlayerHealth < 25.0 ) 
	{
		return kDraw;
	}

	// By default
	fMission.mAdjustDynaverse( kAdjustWarProgression, +1, EnemyTeam->mGetRace() );
	return kVictory;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

	if (PlayerCaptured == 1)
	{
		fMission.mAddToDebriefingList (kPlayerShipCaptured_msg);
	}

	if (PlayerCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kPlayerShipsCaptured_msg);
	}

	if (EnemyCaptured == 1)
	{
		fMission.mAddToDebriefingList (kCapturedSingleEnemy_msg);
	}

	if (EnemyCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kCapturedMultipleEnemies_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
