#include "scriptinterface.h"

#include "Gen_PursuitAsteroidsText.h"
#include "Gen_PursuitAsteroidsMaps.h"

int32 MessageNumber = 0;
bool PirateDestroyed = false;
bool PirateCaptured = false;
bool BaseDestroyed = false;
bool BaseCaptured = false;
bool PlayerDisengaged = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam, kBaseTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kPlayerColonial | kPlayerCapital | kWarzone | kOccupiedColonial | kOccupiedCapital | kAlliedBorder | kUnexplored;
	Specs->fPlayerRace = kNonPirateMission;
	Specs->fEnemyRace = kOrionMission;
	Specs->fNumShips = MissSpecOneShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = 180;
	Specs->fMinDate = 4;
	Specs->fMaxDate = 50;
	Specs->fWait = 4;
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints( 150, 100, 60, 30, 0 );

	// must set up the races of the teams in the script
	info.fEnemyRace = kOrion;
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, kOrion );
	fMission.mSetTeamRace( kBaseTeam, kOrion );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0 );
	fMission.mSetTeamRelations( kPlayerTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kEnemyTeam, kPlayerTeam, kTeamAtWar, kGoalIgnore, 1.0 );
	fMission.mSetTeamRelations( kEnemyTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

	// pick a random map
	return( fMission.mRandomInt32( 1, TotalMaps ));
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );

	if( Mode == 0 )
	{
		int32 DiffLevel = fMission.mGetDifficultyLevel();

		// create pirate ship
		// Easy = 0.75 PBPV
		// Medium = 1.125
		// Hard = 1.5
		int32 TotalBPV = PlayerTeam->mGetCombatBPV() * ( .75 + ( .25 * DiffLevel * 1.5 ));
		EnemyTeam->mCreateShip( TotalBPV, kClassFrigate, kClassHeavyCruiser, 0, kStartPosition_Z );
	}

	if( Mode == 1 )
	{
		// create the pirate base
		BaseTeam->mCreateShip( kMaxBPV, kClassBaseStation, kClassBaseStation, 100, kStartPosition_W );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	MessageNumber = fMission.mRandomInt32( kBriefingA_msg, kBriefingC_msg );

	return( MessageNumber );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );
	tShipInfo* Ship;

	// configure player ship
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mSetShipThrust( 20 );

	// configure pirates
	Ship = EnemyTeam->mGetFirstShip();
	Ship->mSetCommButton( 1, kButtonHail_text );
	Ship->mSetFinalDestination();
	Ship->mSetShipMaxSpeed( 20 );

	fMission.mSetGoalIndicator( Ship );

	if( MessageNumber == kBriefingC_msg )
	{
		EnemyTeam->mGetFirstShip()->mDamageSystem( kDamageWeaponsAndSystems, 25 );
	}

	fMission.mSetTimedInterval( kTimerLimitSet, 1 );
	fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	// The Base
	Ship->mSetCommButton (1, kButtonHail_text);
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* BaseTeam = fMission.mGetTeamHandle( kBaseTeam );

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerTurnsSet:
					mCreateMissionShips( 1 );
					break;

				case kTimerLimitSet:
				 	fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );
					fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0);
					fMission.mSetTeamRelations (kPlayerTeam, kBaseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kEnemyTeam, kPlayerTeam, kTeamAtWar, kGoalIgnore, 1.0);
					fMission.mSetTeamRelations (kEnemyTeam, kBaseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					break;
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetOriginalTeam() == kBaseTeam )
			{
				if( BaseCaptured == false )
				{
					BaseDestroyed = true;
				}

				if( EnemyTeam->mGetShipsInPlayCount() == 0 )
				{
					return kPlayerWins;
				}
			}

			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				if( PirateCaptured == false )
				{
					PirateDestroyed = true;
				}

				if(( BaseTeam->mGetShipsInPlayCount() == 0 ) 
					&& ( EnemyTeam->mGetShipsInPlayCount() == 0 ))
				{
					return kPlayerWins;
				}

				fMission.mDisplayMessage( kShallWe_msg, kScienceScriptMessage );
			}
			break;

		case kNotifyAtDestination:
			fMission.mSetTeamRelations( kEnemyTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0 );
			Ship->mDefendEntity( BaseTeam->mGetFirstShip(), 20, 40, kAbsolutePriority );
			fMission.mDisplayMessage( kPirateTurning_msg, kScienceScriptMessage );
			break;

		case kNotifyOnCapture:
			if( Ship->mGetOriginalTeam() == kBaseTeam )
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
				Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
				fMission.mDisplayMessage( kPirateBaseSelfDestruct_msg, kScienceScriptMessage );
				BaseCaptured = true;
			}

			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
				Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
				fMission.mDisplayMessage( kPirateShipSelfDestruct_msg, kScienceScriptMessage, Ship );
				PirateCaptured = true;
			}
			break;

		case kNotifyCommButtonPress:
			if( Ship->mGetTeam() == kEnemyTeam )
			{
				fMission.mDisplayMessage( kNoResponse_msg, kMissionScriptMessage );
			}
			else
			{ 
				fMission.mDisplayMessage( kNoBase_msg, kMissionScriptMessage );
			}
			break;

		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				PlayerDisengaged = true;
				return kPlayerDisengaged;
			}
			return kPlayerDisengaged;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	if(
			(( PirateCaptured == true ) || ( PirateDestroyed == true ))
		&&  (( BaseDestroyed == true ) || ( BaseCaptured == true ))
	  )
	{
		return kAstoundingVictory;
	}

	if( PlayerDisengaged == true )
	{
		return kDraw;
	}

	return kVictory;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList( VictoryLevel + kAstoundingVictory_msg );

	if( PirateDestroyed == true )
	{
		fMission.mAddToDebriefingList( kPirateDestroyed_msg );
	}

	if( PirateCaptured == true )
	{
		fMission.mAddToDebriefingList( kPirateCaptured_msg );
	}

	if( BaseDestroyed == true )
	{
		fMission.mAddToDebriefingList( kBaseDestroyed_msg );
	}

	if( BaseCaptured == true )
	{
		fMission.mAddToDebriefingList( kBaseCaptured_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
