#include "scriptinterface.h"

#include "Gen_SurveyEnemyText.h"
#include "Gen_SurveyEnemyMaps.h"

int32 BonusBalancePoints = 0;
int32 EnemyCaptured = 0;
int32 PlanetScanned = 0;	// 0 == no, 1 == scanning, 2 == scanned
int32 ProbesRequired = 0;
float PlayerHealth = 0.0;
float EnemyHealth = 0.0;
bool PlayerDisengaged = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam, kPlanetTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kWarzone | kPlayerColonial | kUnexplored;
	Specs->fPlayerRace = kNonPirateMission;
	Specs->fEnemyRace = kNonPirateMission;
	Specs->fNumShips = MissSpecOneShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 2;
	Specs->fMaxDate = 50;
	Specs->fWait = 2;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints( 65, 40, 10, 0, 0 );

	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, info.fEnemyRace );
	fMission.mSetTeamRace( kPlanetTeam, info.fPlayerRace );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kEnemyTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

	// pick a random map
	return( fMission.mRandomInt32( 1, TotalMaps ));
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle( kPlanetTeam );

	// Get the total BPV of the player and modify by 80% to 120%
	int32 TotalBPV = PlayerTeam->mGetCombatBPV();
	int32 NumShips = 1;
	int32 DiffLevel = fMission.mGetDifficultyLevel();	// 0 = easy, 1 = medium, 2 = hard
	ProbesRequired = 1 + DiffLevel;

	if( DiffLevel == 0 )
	{
		TotalBPV = TotalBPV * .7;
	}

	if(( DiffLevel == 1 ) && ( fMission.mRandomInt32( 1, 2 ) == 2 ))
	{
		NumShips++;
	}

	if( DiffLevel == 2 )
	{
		TotalBPV = TotalBPV * 1.4;
		NumShips++;
	}

	EnemyTeam->mCreateFleet( NumShips, TotalBPV, kClassFrigate, PlayerTeam->mGetMaxClassType(), 33, kStartPosition_M + fMission.mRandomInt32( 0, 12 ));

	PlanetTeam->mCreateShip( "PL3", kStartPosition_Specified, 23, 21 );

}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return( fMission.mRandomInt32( kBriefingA_msg, kBriefingC_msg ));
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle( kPlanetTeam );
	tShipInfo* Ship;

	int32 PBPV = PlayerTeam->mGetCombatBPV();
	int32 EBPV = EnemyTeam->mGetCombatBPV();
	BonusBalancePoints = ( 20 * EBPV / PBPV ) - 20;

	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mTrackTacticalDistance( 30.0, 23, 21 );
	Ship->mSetShipThrust( 5 );

	for( Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip() )
	{
		Ship->mSetCommButton( 1, kButtonHail_text );
	}

	PlanetTeam->mGetFirstShip()->mSetShipAIControlled( kFalse );

	fMission.mSetTimedInterval( kTimerLimitSet, 2 );
	fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle( kPlanetTeam );

	PlayerHealth = 100.0f - PlayerTeam->mGetTotalDamagePercent();
	EnemyHealth = 100.0f - EnemyTeam->mGetTotalDamagePercent();

	int32 Enemies = EnemyTeam->mGetShipsInPlayCount();

	switch( Event )
	{
		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				return kPlayerDestroyed;
			}
			else
			{
				if(( Enemies == 0 ) && ( PlanetScanned == 2 ))
				{
					fMission.mAddToBonusPrestige( BonusBalancePoints );
					return kEnemyDestroyed;
				}
				else if( Enemies == 0 )
				{
					fMission.mDisplayMessage( kNoMoreEnemy_msg, kScienceScriptMessage );
				}
			}
			break;

		case kNotifyOnCapture:
			EnemyCaptured++;
			fMission.mAddToBonusPrestige( kCaptureBonus );

			if(( Enemies == 1 ) && ( PlanetScanned == 2 ))
			{
				fMission.mAddToBonusPrestige( BonusBalancePoints );
				return kEnemyCaptured;
			}
			else if( Enemies == 1 )
			{
				fMission.mDisplayMessage( kNoMoreEnemy_msg, kScienceScriptMessage );
			}
			break;

		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				PlayerDisengaged = true;
				fMission.mAddToBonusPrestige( BonusBalancePoints );
				return kPlayerWins;
			}
			break;

		case kNotifyAtDistance:
			if( fIsDeepScanOn == kTrue )
			{
				fMission.mSetTimedInterval( kTimerTurnsSet_2, 1 );
				fMission.mDisplayMessage( kScanningPlanet_msg, kScienceScriptMessage );
				PlanetScanned = 1;
			}
			else
			{
				fMission.mDisplayMessage( kCloseEnough_msg, kScienceScriptMessage );
			}
			break;

		case kNotifyCommButtonPress:
			fMission.mDisplayMessage( kEnemyHail_msg, kMissionScriptMessage );

			for( Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip() )
			{
				Ship->mClearCommButton( 1 );
			}
			break;

		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kEnemyTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );
					break;

				case kTimerTurnsSet:
					fMission.mDisplayMessage( kScienceBrief_msg, kScienceScriptMessage );
					break;

				case kTimerTurnsSet_2:
					if( fIsDeepScanOn == kTrue )
					{
						if( fPlayerShip->mGetTacticalDistance( 23, 21 ) <= 30.0 )
						{
							fMission.mDisplayMessage (kPlanetScanned_msg, kScienceScriptMessage);
						}
						else
						{
							fMission.mDisplayMessage (kFarButGotIt_msg, kScienceScriptMessage);
						}

						PlanetScanned = 2;
						fMission.mAddToBonusPrestige( 60 );

						if( EnemyTeam->mGetShipsInPlayCount() == 0 )
						{
							fMission.mAddToBonusPrestige( BonusBalancePoints );
							return kEnemyDestroyed;
						}
					}
					break;
			}
			break;

		case kNotifyDeepScan:
			if( fIsDeepScanOn == kTrue )
			{
				if( PlanetScanned == 0 )
				{
					if( fPlayerShip->mGetTacticalDistance( 23, 21 ) <= 30.0 )
					{
						fMission.mSetTimedInterval( kTimerTurnsSet_2, 1 );
						fMission.mDisplayMessage( kScanningPlanet_msg, kScienceScriptMessage );
						PlanetScanned = 1;
					}
					else
					{
						fMission.mDisplayMessage( kTooFar_msg, kScienceScriptMessage );
					}
				}
			}
			else if( PlanetScanned == 1 )
			{
				fMission.mDisplayMessage (kDidNotScan_msg, kScienceScriptMessage);
				PlanetScanned = 0;
				fMission.mSetTimedInterval( kTimerTurnsSet_2, 0 );
			}
			break;

		case kNotifyProbeLaunch:
			Ship = PlanetTeam->mGetFirstShip();
			if(( PlanetScanned != 2 ) && ( fMission.mGetShipTarget( fPlayerShip ) == Ship ))
			{
				ProbesRequired--;

				if( ProbesRequired == 0 )
				{
					fMission.mDisplayMessage( kPlanetScanned_msg, kScienceScriptMessage );
					PlanetScanned = 2;
					fMission.mAddToBonusPrestige( 60 );

					if( EnemyTeam->mGetShipsInPlayCount() == 0 )
					{
						fMission.mAddToBonusPrestige( BonusBalancePoints );
						return kEnemyDestroyed;
					}
				}
				else
				{
					fMission.mDisplayMessage( kMoreProbes_msg, kScienceScriptMessage );
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	if(( PlayerDisengaged == true ) && ( PlanetScanned != 2 ))
	{
		if( PlayerHealth > EnemyHealth )
		{
			return kVictory;
		}
		else if( PlayerHealth < ( EnemyHealth / 2 ))
		{
			return kDevastatingDefeat;
		}
		else 
		{
			return kDefeat;
		}
	}

	if(( PlayerHealth > 90.0 ) || ( EnemyCaptured ))
	{
		return kAstoundingVictory;
	}

	if( PlayerHealth < 25.0 ) 
	{
		return kDraw;
	}

	// By default
	return kVictory;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if( PlanetScanned == 2 )
	{ 
		fMission.mAddToDebriefingList( kDebriefScanned_msg );
	}
	else
	{
		fMission.mAddToDebriefingList( kDebriefNoScan_msg );
	}

	fMission.mAddToDebriefingList( VictoryLevel + kAstoundingVictory_msg );

	if( EnemyCaptured == 1 )
	{
		fMission.mAddToDebriefingList( kCapturedSingleEnemy_msg );
	}

	if( EnemyCaptured >= 2 )
	{
		fMission.mAddToDebriefingList( kCapturedMultipleEnemies_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
