#include "scriptinterface.h"

#include "Gor_3MissionText.h"
#include "Gor_3MissionMaps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 Elite = 0;
int32 PlayerDestroyed = 0;
int32 PatrolDestroyed = 0;
int32 ColonyDestroyed = 0;
int32 EnemyDestroyed = 0; // total amount of Patrol and Colony ships destroyed
int32 EnemyCount = 0; // total number of Patrol and Colony ships
int32 OfferStatus = 0;
int32 UpgradeReceived = 0; // win condition
int32 PlanetScanned = 0;
int32 PlanetBusted = 0; // win condition

bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key
bool PlanetIndestructible = false;  // shield raised
bool CloseToPlanet = false;

const float kDistanceToUpgradePlanet = 20.0f;
const int32 kUpgradePlanetX = 11;
const int32 kUpgradePlanetY = 12;
const float kColonyThreatDistance = 45.0f;
const float kColonyFollowDistance = 60.0f;
const int32 kRaisingShieldTimer = 10; // number of turns for planet to raise shields
									  // this value will be divided by two, so it'd be good if it were evenly divisible by 2
const float kPlanetDamage = 20.0f;
const float kDeepScanDistancePlanet = 30.0f;
const int32 kDeepScanTurnsPlanet = 2;
const float kFedDamagePercent = 60.0f;
const float kColonyDestroyedDamage = 30.0f;

void OrbitPlanet(tScript& tScript, tTeamInfo* OrbitTeam);
void RomulanCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship);
void FederationCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship);
eGameStatus CheckWinCondition( tScript& tScript );

enum eEliteOrganizations
{
	kKingsGuard = 1,
	kEggDefender,
	kErrant
};

enum eOfferStatus
{
	kNoOffer = 0,
	kMadeOffer,
	kAcceptedOffer,
	kRejectedOffer,
	kFinalizedOffer
};

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kColonyTeam, 
	kPatrolTeam,
	kPlanetTeam
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kGornMission;
	Specs->fEnemyRace = kRomulanMission | kFedMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (300, 240, 150, 60, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = (fMission.mRandomInt32 (1, 100) < 50) ? kRomulan : kFederation;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kColonyTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kPatrolTeam, info.fEnemyRace);
	fMission.mSetTeamRace (kPlanetTeam, kGorn);

	// Set team relations to show up on Tactical map
	// this mission starts without any enemy ships on screen
	fMission.mSetTeamRelations (kPlayerTeam, kColonyTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPatrolTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kColonyTeam, kPatrolTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kColonyTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kPatrolTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	if (fMission.mGetEliteOrganization() == kKingsGuard)
	{
		Elite = kKingsGuard;
	}
	else if (fMission.mGetEliteOrganization() == kEggDefender)
	{
		Elite = kEggDefender;
	}
	else
	{
		Elite = kErrant;
	}

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* ColonyTeam = fMission.mGetTeamHandle (kColonyTeam);
	tTeamInfo* PatrolTeam = fMission.mGetTeamHandle (kPatrolTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (tShipInfo* Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;

		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;

		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	// create planets
	PlanetTeam->mCreateShip( "PLPh1", kStartPosition_Y );
	PlanetTeam->mCreateShip( "PL2", kStartPosition_Z );

	if ( ColonyTeam->mGetRace() == kFederation )
	{
		// create Colony team
		ColonyTeam->mCreateShip( TotalBPV, kClassDestroyer, kClassDestroyer, 0, kStartPosition_X );
		EnemyCount += 1;

		// create Patrol team
		PatrolTeam->mCreateShip( AveragePlayerBPV, kClassFrigate, kClassFrigate, 0, kStartPosition_V );
		EnemyCount += 1;
	}
	else
	{
		// create Colony team
		ColonyTeam->mCreateShip( AveragePlayerBPV, kClassDestroyer, kClassDestroyer, 0, kStartPosition_X );
		EnemyCount += 1;

		// create Patrol team
		PatrolTeam->mCreateShip( TotalBPV, kClassDestroyer, kClassDreadnought, 40, kStartPosition_U );
		//PatrolTeam->mCreateShip( AveragePlayerBPV, kClassFrigate, kClassFrigate, 0, kStartPosition_V );
		EnemyCount += 1;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	tTeamInfo* ColonyTeam = fMission.mGetTeamHandle (kColonyTeam);

	if ( Elite == kKingsGuard )
	{
		if ( ColonyTeam->mGetRace() == kRomulan )
		{
			return( kKingsGuardBriefingRomulan_msg );
		}
		else
		{
			return( kKingsGuardBriefingFed_msg );
		}
	}
	else if ( Elite == kEggDefender )
	{
		if ( ColonyTeam->mGetRace() == kRomulan )
		{
			return( kEggDefenderBriefingRomulan_msg );
		}
		else
		{
			return( kEggDefenderBriefingFed_msg );
		}
	}
	else
	{
		if ( ColonyTeam->mGetRace() == kRomulan )
		{
			return( kErrantBriefingRomulan_msg );
		}
		else
		{
			return( kErrantBriefingFed_msg );
		}
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* ColonyTeam = fMission.mGetTeamHandle (kColonyTeam);
	tTeamInfo* PatrolTeam = fMission.mGetTeamHandle (kPatrolTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mTrackTacticalDistance( kDistanceToUpgradePlanet, kUpgradePlanetX, kUpgradePlanetY );
		Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
	}

	// configure the planets
	PlanetTeam->mGetThisShip( 1 )->mChangeName( kPlanetGarragan2_name );
	PlanetTeam->mGetFirstShip()->mDamageSystem( kDamageMaxShields, 100.0f );
	PlanetTeam->mGetThisShip( 2 )->mChangeName( kPlanetGarragan_name );
	PlanetTeam->mGetFirstShip()->mSetCommButton (1, kButtonHail_text );
	PlanetTeam->mGetFirstShip()->mSetShipCallbackEvents( kNotifyDamage );
	for (Ship = PlanetTeam->mGetFirstShip(); Ship != NULL; Ship = PlanetTeam->mGetNextShip())
	{
	}

	// configure Colony team
	for (Ship = ColonyTeam->mGetFirstShip(); Ship != NULL; Ship = ColonyTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		Ship->mDefendEntity( PlanetTeam->mGetFirstShip(), kColonyThreatDistance, kColonyFollowDistance, kVeryHighPriority );
	}

	// configure Patrol team
	for (Ship = PatrolTeam->mGetFirstShip(); Ship != NULL; Ship = PatrolTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
	}
	OrbitPlanet( *this, PatrolTeam );

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer to set team relations
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// timer to bring down planet's shields
	fMission.mSetTimedInterval (kTimerLimitSet_2, 10);

	// setup to test deep scan every second - must be called before any entities are added to be scanned
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	// deep scan ListeningPost from within kDeepScanDistance for 1 turn
	DeepScanTracker.mSetTarget (PlanetTeam->mGetThisShip(1), kDeepScanDistancePlanet, kDeepScanTurnsPlanet);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* ColonyTeam = fMission.mGetTeamHandle (kColonyTeam);
	tTeamInfo* PatrolTeam = fMission.mGetTeamHandle (kPatrolTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyTransportedItemTo:
			{
				tShipInfo* TransportTarget = fMission.mFindShipID( fEventID );

				if ( TransportTarget != NULL )
				{
					if ( Ship->mGetOriginalTeam() != kPlayerTeam )
					{
						if ( fEventItem1 == kTransWeaponSchematic )
						{
							Ship->mAccessSupplies( kSetTransporterItemCount, kTransWeaponSchematic, 1 );
							TransportTarget->mAccessSupplies( kSetTransporterItemCount, kTransWeaponSchematic, 0 );
							Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
						}
					}
				}
			}
			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID( fEventID );

				if ( HostileShip != NULL )
				{
					if ( HostileShip->mGetTeam() == kPlayerTeam )
					{
						if ( Ship->mGetTeam() == kColonyTeam )
						{
							PatrolTeam->mClearCommands();
							fMission.mSetTeamRelations (kPlayerTeam, kPatrolTeam, kTeamHostile, kGoalDestroy, 1.0);
							fMission.mSetTeamRelations (kPatrolTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);
						}
						else if ( Ship->mGetTeam() == kPlanetTeam )
						{
							if ( PlanetIndestructible == true )
							{
								PlanetTeam->mGetFirstShip()->mDamageSystem( kDamageWeaponsSystemsAndShields, 0 );
							}
							else
							{
								if ( Ship == PlanetTeam->mGetThisShip( 1 ) )
								{
									// LNF maybe this stuff should be only in COMM stuff, like Surrender? Or Threaten if < 20 kkm?
									// LNF it doesn't really make sense if the player is ALREADY shooting at the planet
									if ( OfferStatus == kNoOffer )
									{
										OfferStatus = kMadeOffer;
										fMission.mDisplayMessage( kEvent2Offer_1_msg, kMissionScriptMessage );
										fMission.mDisplayMessage( kEvent2Offer_2_msg, kMissionScriptMessage );
										fMission.mDisplayMessage( kEvent2Offer_3_msg, kMissionScriptMessage );

										Ship->mSetCommButton (1, kButtonAccept_text);
										Ship->mSetCommButton (2, kButtonReject_text);
									}
									else if ((( Ship->mGetDamagePercent() >= kPlanetDamage )
											&& ( Ship->mGetDamagePercent() < kColonyDestroyedDamage ))
											&& ( OfferStatus == kMadeOffer ))
									{
										// reject the offer here
										OfferStatus = kRejectedOffer;
										PlanetTeam->mGetFirstShip()->mSetCommButton (1, kButtonHail_text);
									}
									else if (( Ship->mGetDamagePercent() >= kColonyDestroyedDamage )
											&& ( PlanetBusted == 0 ))
									{
										// settlements should be destroyed and the planet considered secure
										fMission.mDisplayMessage( kColonyTakenCareOf_msg, kScienceScriptMessage );
										PlanetBusted = 1;
										return CheckWinCondition( *this );
									}
								}
							}
						}
					}
					Ship->mSetShipCallbackEvents( kNotifyDamage );
				}
			}
			break;

		case kNotifyAtDistance:
			if ( Ship->mGetTrackingDistance() == kDistanceToUpgradePlanet )
			{
				CloseToPlanet = true;
				fMission.mDisplayMessage( kEvent1InRange_msg, kScienceScriptMessage );

				fMission.mSetTimedInterval( kTimerTurnsSet, (kRaisingShieldTimer / 2) );
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet: // set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kColonyTeam, kTeamHostile, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kPlayerTeam, kPatrolTeam, kTeamHostile, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kColonyTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kColonyTeam, kPatrolTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kColonyTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kPatrolTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kPatrolTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mDisplayMessage( kGarrigan2ahead_msg, kMissionScriptMessage );
					break;

				case kTimerLimitSet_2:
					// bring down planet's shields
					if ( PlanetIndestructible == false )
					{
						PlanetTeam->mGetFirstShip()->mDamageSystem( kDamageMaxShields, 100.0f );
						fMission.mSetTimedInterval( kTimerLimitSet_2, 10 );
					}
					break;
					
				case kTimerLimitSet_10:
					{
						eDeepScanFlags flag;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag);
						switch (flag)
						{
							case kOutOfRange_ScanOff:
								// player is not within any objects range and deep scan is off (Object == NULL)
								break;
							case kOutOfRange_ScanOn:
								// player is not within any objects range and deep scan is on (Object == NULL)
								break;
							case kEnteringRange_ScanOff:
								// player has just entered range and scan is off (Object points to entity)
								// notify player to turn scan on
								if (( Object != NULL ) && ( Object->mGetTeam() == kPlanetTeam ))
								{
									//fMission.mDisplayMessage (kScannerOffInRangePlanet_msg, kScienceScriptMessage);
								}
								else if ( Object != NULL )
								{
									//fMission.mDisplayMessage (kScannerOffInRangeLp_msg, kScienceScriptMessage);
								}
								
								break;
							case kEnteringRange_ScanOn_TimerStarted:
								// player has just entered range and scan is on (Object points to entity)
								// notify player that timer has begun
								if (( Object != NULL ) && ( Object->mGetTeam() == kPlanetTeam ))
								{
									//fMission.mDisplayMessage (kEvent4BeginScanningPlanet_msg, kScienceScriptMessage);
								}
								else if ( Object != NULL )
								{
									//fMission.mDisplayMessage (kEvent1A1_msg, kScienceScriptMessage);
								}
								break;
							case kInRange_ScanOff_TimerStopped:
								// player is in range and scan is off (Object points to entity)
								// notify player that timer has been turned off
								//fMission.mDisplayMessage (kTurnedOffScanInRange_msg, kScienceScriptMessage);
								break;
							case kInRange_ScanOff:
								// player is still in range and scan is still off (Object points to entity)
								break;
							case kInRange_ScanOn_TimerStarted:
								// player is in range and scan is on (Object points to entity)
								// notify player that timer has begun
								if (( Object != NULL ) && ( Object->mGetTeam() == kPlanetTeam ))
								{
									//fMission.mDisplayMessage (kEvent4BeginScanningPlanet_msg, kScienceScriptMessage);
								}
								else if ( Object != NULL )
								{
									//fMission.mDisplayMessage (kEvent1A1_msg, kScienceScriptMessage);
								}
								break;
							case kInRange_ScanOn:
								// player is still in range and scan is still on (Object points to entity)
								break;
							case kLeavingRange_ScanOn_TimerStopped:
								// player has just left the range and scan is on (Object points to entity)
								// notify player that timer is now off
								//fMission.mDisplayMessage (kTurnedOffScanInRange_msg, kScienceScriptMessage);
								break;
							case kLeavingRange_ScanOff:
								// player has just left the range and scan is off (Object points to entity)
								break;
						}
					}
					break;

				case kTimerCountdownSet:
					{
						eDeepScanFlags flag;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag);
						if (flag == kInRange_ScanOn)
						{
							// player has successfully scanned the object
							if (( Object != NULL ) && ( Object->mGetOriginalTeam() == kPlanetTeam ))
							{
								PlanetScanned = 1;

								if ( OfferStatus == kNoOffer )
								{
									fMission.mDisplayMessage( kEvent6ScanPlanet_msg, kScienceScriptMessage );
								}
								else if ( OfferStatus == kMadeOffer )
								{
									fMission.mDisplayMessage( kEvent6ScanPlanet_msg, kScienceScriptMessage );
								}
								else if (( OfferStatus == kAcceptedOffer )
										|| ( OfferStatus == kRejectedOffer ))
								{
									fMission.mDisplayMessage( kEvent6ScanPlanet_msg, kScienceScriptMessage );
								}
							}

							DeepScanTracker.mRemoveTarget (Object);
						}
					}
					break;

				case kTimerTurnsSet:
					// warning that the shields will come online
					fMission.mDisplayMessage( kShieldAlmostThere_msg, kScienceScriptMessage );
					fMission.mSetTimedInterval( kTimerTurnsSet_2, (kRaisingShieldTimer / 2) );
					break;

				case kTimerTurnsSet_2:
					if ( PlanetBusted == 0 )
					{
						// Planet becomes indestructible
						fMission.mDisplayMessage( kEvent3Shields_msg, kScienceScriptMessage );
						PlanetIndestructible = true;

						// LNF there is a (negative) victory condition here
					}
					break;

				case kTimerTurnsSet_8:
					// set Planet comm buttons back to HAIL
					for (Ship = PlanetTeam->mGetFirstShip(); Ship != NULL; Ship = PlanetTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set Patrol ships' comm buttons back to HAIL
					for (Ship = PatrolTeam->mGetFirstShip(); Ship != NULL; Ship = PatrolTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set Colony Team comm buttons back to HAIL
					for (Ship = ColonyTeam->mGetFirstShip(); Ship != NULL; Ship = ColonyTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyOnCapture:

			fMission.mAddToBonusPrestige( kCaptureBonus );

			if (Ship->mGetOriginalTeam() == kPatrolTeam)
			{
				PatrolDestroyed++;
				EnemyDestroyed++;

				if (PatrolTeam->mGetShipsInPlayCount() == 0)
				{
					if ( ColonyTeam->mGetShipsInPlayCount() == 0 )
					{
						// LNF message from Governor
						// also need to check if captured or disengaged
						fMission.mDisplayMessage( kEvent9AllShipsDestroyed_msg, kScienceScriptMessage );
						return CheckWinCondition( *this );
					}
				}
			}
			else if (Ship->mGetOriginalTeam() == kColonyTeam)
			{
				ColonyDestroyed++;
				EnemyDestroyed++;

				if (ColonyTeam->mGetShipsInPlayCount() == 0)
				{
					if (PatrolTeam->mGetShipsInPlayCount() == 0)
					{
						// LNF message from Governor
						// also need to check if captured or disengaged
						fMission.mDisplayMessage( kEvent9AllShipsDestroyed_msg, kScienceScriptMessage );
						return CheckWinCondition( *this );
					}
				}
			}
			else if ( Ship->mGetTeam() == kPlanetTeam )
			{
				if ( Ship == PlanetTeam->mGetFirstShip() )
				{
					PlanetBusted = 1;
					fMission.mDisplayMessage( kColonyCaptured_msg, kScienceScriptMessage );
					return CheckWinCondition( *this );
				}
				else
				{
				}
			}
			break;

		case kNotifyAtDeath:
			if (Ship->mGetOriginalTeam() == kPatrolTeam)
			{
				if ( Ship->mTestShipStatus( kShipCaptured ) ==  kFalse )
				{
					PatrolDestroyed++;
					EnemyDestroyed++;
				}

				if (PatrolTeam->mGetShipsInPlayCount() == 0)
				{
					if ( ColonyTeam->mGetShipsInPlayCount() == 0 )
					{
						// LNF message from Governor
						// also need to check if captured or disengaged
						fMission.mDisplayMessage( kEvent9AllShipsDestroyed_msg, kScienceScriptMessage );
						return CheckWinCondition( *this );
					}
				}
			}
			else if (Ship->mGetOriginalTeam() == kColonyTeam)
			{
				if ( Ship->mTestShipStatus( kShipCaptured ) ==  kFalse )
				{
					ColonyDestroyed++;
					EnemyDestroyed++;
				}

				if (ColonyTeam->mGetShipsInPlayCount() == 0)
				{
					if (PatrolTeam->mGetShipsInPlayCount() == 0)
					{
						// LNF message from Governor
						// also need to check if captured or disengaged
						fMission.mDisplayMessage( kEvent9AllShipsDestroyed_msg, kScienceScriptMessage );
						return CheckWinCondition( *this );
					}
				}
			}
			else if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			break;

		case kNotifyDisengaged:
			// LNF make sure to check all this (capture, death, disengage) for logic errors

			if (Ship->mGetOriginalTeam() == kPatrolTeam)
			{
				if ( Ship->mTestShipStatus( kShipCaptured ) ==  kFalse )
				{
					PatrolDestroyed++;
					EnemyDestroyed++;
				}

				if (PatrolTeam->mGetShipsInPlayCount() == 0)
				{
					if ( ColonyTeam->mGetShipsInPlayCount() == 0 )
					{
						// LNF message from Governor
						// also need to check if captured or disengaged
						return CheckWinCondition( *this );
					}
				}
			}
			else if (Ship->mGetOriginalTeam() == kColonyTeam)
			{
				if ( Ship->mTestShipStatus( kShipCaptured ) ==  kFalse )
				{
					ColonyDestroyed++;
					EnemyDestroyed++;
				}

				if (ColonyTeam->mGetShipsInPlayCount() == 0)
				{
					if (PatrolTeam->mGetShipsInPlayCount() == 0)
					{
						// LNF message from Governor
						// also need to check if captured or disengaged
						return CheckWinCondition( *this );
					}
				}
			}
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonAccept_text:
						OfferStatus = kFinalizedOffer;
						UpgradeReceived = 1;

						fMission.mDisplayMessage( kEvent5GettingSchematic_1_msg, kMissionScriptMessage );
						fMission.mDisplayMessage( kEvent5GettingSchematic_2_msg, kMissionScriptMessage );
						//fMission.mDisplayMessage( kEvent5GettingSchematic_3_msg, kMissionScriptMessage );

						fPlayerShip->mAccessSupplies( kSetTransporterItemCount, kTransWeaponSchematic, 1 );

						PlanetTeam->mGetFirstShip()->mSetCommButton (1, kButtonHail_text);
						PlanetTeam->mGetFirstShip()->mClearCommButton(2);
						break;

					case kButtonReject_text:
						OfferStatus = kRejectedOffer;
						PlanetTeam->mGetFirstShip()->mSetCommButton (1, kButtonHail_text);
						break;

					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kPatrolTeam:
								( PatrolTeam->mGetRace() == kRomulan ) ? RomulanCommMessages(*this, fEventID, Ship)
									: FederationCommMessages(*this, fEventID, Ship);

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kColonyTeam:
								( ColonyTeam->mGetRace() == kRomulan ) ? RomulanCommMessages(*this, fEventID, Ship)
									: FederationCommMessages(*this, fEventID, Ship);

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kPlanetTeam:
								if ( Ship == PlanetTeam->mGetFirstShip() )
								{
									if ( PlanetIndestructible == false )
									{
										if ( OfferStatus == kNoOffer )
										{
											if ( CloseToPlanet == false )
											{
												fMission.mDisplayMessage( kPlanetHail_msg, kMissionScriptMessage );
											}
											else
											{
												// make the offer for weapons schematic
												OfferStatus = kMadeOffer;
												fMission.mDisplayMessage( kEvent2Offer_1_msg, kMissionScriptMessage );
												fMission.mDisplayMessage( kEvent2Offer_2_msg, kMissionScriptMessage );
												fMission.mDisplayMessage( kEvent2Offer_3_msg, kMissionScriptMessage );

												Ship->mSetCommButton (1, kButtonAccept_text);
												Ship->mSetCommButton (2, kButtonReject_text);
												break;
											}
										}
										else if ( OfferStatus == kAcceptedOffer )
										{
											Ship->mSetCommButton (1, kButtonAccept_text);
											Ship->mSetCommButton (2, kButtonReject_text);
											break;
										}
										else // if OfferStatus > kAcceptedOffer
										{
											fMission.mDisplayMessage( kNoResponse_msg, kMissionScriptMessage );
										}
									}
									else // PlanetIndestructible == true
									{
										fMission.mDisplayMessage( kPlanetHailShields_msg, kMissionScriptMessage );
									}
								}
								else
								{
								}

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kPatrolTeam:
								( PatrolTeam->mGetRace() == kRomulan ) ? RomulanCommMessages(*this, fEventID, Ship)
									: FederationCommMessages(*this, fEventID, Ship);

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kColonyTeam:
								( ColonyTeam->mGetRace() == kRomulan ) ? RomulanCommMessages(*this, fEventID, Ship)
									: FederationCommMessages(*this, fEventID, Ship);

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kPlanetTeam:

								if ( UpgradeReceived == 0 )
								{
									fMission.mDisplayMessage( kPlanetSurrender_msg, kMissionScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage( kNoResponse_msg, kMissionScriptMessage );
								}

								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kPatrolTeam:
								( PatrolTeam->mGetRace() == kRomulan ) ? RomulanCommMessages(*this, fEventID, Ship)
									: FederationCommMessages(*this, fEventID, Ship);

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kColonyTeam:
								( ColonyTeam->mGetRace() == kRomulan ) ? RomulanCommMessages(*this, fEventID, Ship)
									: FederationCommMessages(*this, fEventID, Ship);

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kPlanetTeam:
								// LNF should this be destroyed/surrendered instead of upgradereceived?
								if ( UpgradeReceived == 0 )
								{
									fMission.mDisplayMessage( fMission.mRandomInt32( kPlanetTaunt_1_msg, kPlanetTaunt_4_msg ), kMissionScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage( kNoResponse_msg, kMissionScriptMessage );
								}

								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_8, 2);
								break;
						}
						break;
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	// LNF adjust the PatrolDestroyed and ColonyDestroyed points depending on how
	// LNF many ships are on those repective teams
	VictoryPoints = (PatrolDestroyed * 20) + (ColonyDestroyed * 20);

	if ( Elite == kKingsGuard )
	{
		VictoryPoints += (UpgradeReceived * 10) + (PlanetBusted * 50);
	}
	else if ( Elite == kEggDefender )
	{
		VictoryPoints += (UpgradeReceived * 20) + (PlanetBusted * 40);
	}
	else // Elite == kErrant
	{
		VictoryPoints += (UpgradeReceived * 40) + (PlanetBusted * 20);
	}

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if (VictoryPoints <= 49)
	{
		MyVictoryLevel = kDefeat;
	}

	// remove the schematic item from the player's transporter
	for (tShipInfo* Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mAccessSupplies( kSetTransporterItemCount, kTransWeaponSchematic, 0 );
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		if (fMission.mGetEliteOrganization() == kKingsGuard)
		{
			fMission.mAddToDebriefingList (VictoryLevel + kKingsGuardAstoundingVictory_msg);
		}
		else if (fMission.mGetEliteOrganization() == kEggDefender)
		{
			fMission.mAddToDebriefingList (VictoryLevel + kEggDefenderAstoundingVictory_msg);
		}
		else
		{
			fMission.mAddToDebriefingList (VictoryLevel + kErrantAstoundingVictory_msg);
		}

		fMission.mAddToDebriefingList (kDismissedShort_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}


	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( UpgradeReceived != 0 ) ? fMission.mAddToDebriefingList( kUpgradeReceivedTrue_msg ) : SuccessCount++;
	( EnemyDestroyed == EnemyCount ) ? fMission.mAddToDebriefingList( kEnemyDestroyedTrue_msg ) : SuccessCount++;
	( PlanetBusted != 0 ) ? fMission.mAddToDebriefingList( kPlanetBustedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 3 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( UpgradeReceived == 0 ) ? fMission.mAddToDebriefingList( kUpgradeReceivedFalse_msg ) : FailureCount++;
	( EnemyDestroyed != EnemyCount ) ? fMission.mAddToDebriefingList( kEnemyDestroyedFalse_msg ) : FailureCount++;
	( PlanetBusted == 0 ) ? fMission.mAddToDebriefingList( kPlanetBustedFalse_msg ) : FailureCount++;
	if ( FailureCount == 3 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}


	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// make the ships orbit the planet
void OrbitPlanet(tScript& tScript, tTeamInfo* OrbitTeam)
{
	tTeamInfo* ColonyTeam = tScript.fMission.mGetTeamHandle (kColonyTeam);
	tTeamInfo* PatrolTeam = tScript.fMission.mGetTeamHandle (kPatrolTeam);
	tShipInfo* Ship;

	const float kPatrolOrbitDistance = 60.0;
	const float kColonyOrbitDistance = 50.0;

	tMapPoint waypoints_1[9]=
	{
		2, 15,
		4, 18,
		9, 20,
		14, 20,
		18, 16,
		17, 11,
		14, 8,
		11, 7,
		6, 9
	};

	tMapPoint waypoints_2[2]=
	{
		10, 14,
		17, 14
	};

	tMapPoint PatrolPoints[10];

	if ( OrbitTeam == PatrolTeam )
	{
		for (Ship = OrbitTeam->mGetFirstShip(); Ship != NULL; Ship = OrbitTeam->mGetNextShip())
		{
			PatrolPoints[0] = waypoints_1[0];
			PatrolPoints[1] = waypoints_1[1];
			PatrolPoints[2] = waypoints_1[2];
			PatrolPoints[3] = waypoints_1[3];
			PatrolPoints[4] = waypoints_1[4];
			PatrolPoints[5] = waypoints_1[5];
			PatrolPoints[6] = waypoints_1[6];
			PatrolPoints[7] = waypoints_1[7];
			PatrolPoints[8] = waypoints_1[8];
			Ship->mPatrol( kPatrolOrbitDistance, 9, &PatrolPoints[0], kHighPriority );
		}
	}
	else if ( OrbitTeam == ColonyTeam )
	{
		for (Ship = OrbitTeam->mGetFirstShip(); Ship != NULL; Ship = OrbitTeam->mGetNextShip())
		{
			PatrolPoints[0] = waypoints_2[0];
			PatrolPoints[1] = waypoints_2[1];
			Ship->mPatrol( kColonyOrbitDistance, 2, &PatrolPoints[0], kHighPriority );
		}
	}
}

//---------------------------------------------------------------------------------------------
// Display comm messages for Romulan
void RomulanCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship)
{
	tTeamInfo* PlayerTeam = tScript.fMission.mGetTeamHandle (tScript.kPlayerTeam);
	tTeamInfo* ColonyTeam = tScript.fMission.mGetTeamHandle (kColonyTeam);
	tTeamInfo* PatrolTeam = tScript.fMission.mGetTeamHandle (kPatrolTeam);


	if ( fEventID == kButtonHail_text )
	{
		tScript.fMission.mDisplayMessage( kRomulanHail_msg, kMissionScriptMessage );
	}
	else if ( fEventID == kButtonSurrender_text )
	{
		tScript.fMission.mDisplayMessage( kRomulanSurrender_msg, kMissionScriptMessage );
	}
	else if ( fEventID == kButtonTaunt_text )
	{
		tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kRomulanTaunt_1_msg, kRomulanTaunt_4_msg), kMissionScriptMessage);
	}
}

//---------------------------------------------------------------------------------------------
// Display comm messages for Federation
void FederationCommMessages(tScript& tScript, int32 fEventID, tShipInfo* Ship)
{
	tTeamInfo* PlayerTeam = tScript.fMission.mGetTeamHandle (tScript.kPlayerTeam);
	tTeamInfo* ColonyTeam = tScript.fMission.mGetTeamHandle (kColonyTeam);
	tTeamInfo* PatrolTeam = tScript.fMission.mGetTeamHandle (kPatrolTeam);


	if ( fEventID == kButtonHail_text )
	{
		if ( Ship->mGetDamagePercent() <= kFedDamagePercent )
		{
			tScript.fMission.mDisplayMessage( kFedHailOK_msg, kMissionScriptMessage );
		}
		else
		{
			tScript.fMission.mDisplayMessage( kFedHailDamaged_msg, kMissionScriptMessage );
		}
	}
	else if ( fEventID == kButtonSurrender_text )
	{
		if ( Ship->mGetDamagePercent() <= kFedDamagePercent )
		{
			tScript.fMission.mDisplayMessage( kFedSurrenderOK_msg, kMissionScriptMessage );
		}
		else
		{
			tScript.fMission.mDisplayMessage( kFedSurrenderDamaged_msg, kMissionScriptMessage );

			// LNF do I need to check if the player is shot at after agreeing to surrender?
			tScript.fMission.mSetTeamRelations ( tScript.kPlayerTeam, Ship->mGetTeam(), kTeamNeutral, kGoalIgnore, 1.0, kTrue );
			Ship->mDisengage( kByMapEdge, kAbsolutePriority );
		}
	}
	else if ( fEventID == kButtonTaunt_text )
	{
		// LNF do I need to check if the Fed ship surrendered, then play a different message?
		tScript.fMission.mDisplayMessage (tScript.fMission.mRandomInt32 (kFedTaunt_1_msg, kFedTaunt_4_msg), kMissionScriptMessage);
	}
}

//---------------------------------------------------------------------------------------------
// Check the win conditions for this mission
eGameStatus CheckWinCondition( tScript& tScript )
{
	tTeamInfo* PlayerTeam = tScript.fMission.mGetTeamHandle (tScript.kPlayerTeam);
	tTeamInfo* ColonyTeam = tScript.fMission.mGetTeamHandle (kColonyTeam);
	tTeamInfo* PatrolTeam = tScript.fMission.mGetTeamHandle (kPatrolTeam);
	tTeamInfo* PlanetTeam = tScript.fMission.mGetTeamHandle (kPlanetTeam);

	if (( PatrolTeam->mGetShipsInPlayCount() == 0 )
		&& ( ColonyTeam->mGetShipsInPlayCount() == 0 )
		&& ( UpgradeReceived == 1 )
		&& ( PlanetBusted == 1 ))
	{
		return kPlayerWins;
	}
	else
	{
		return kGameInProgress;
	}
}