#include "scriptinterface.h"

#include "Hyd_5MissionText.h"
#include "Hyd_5MissionMaps.h"


int32 PlayerDestroyed = 0;
int32 HunterDestroyed = 0;
int32 MonsterDestroyed = 0;
int32 AtWarWithHunter = 0;
int32 TalkedWithNumber5 = 0;
int32 MonsterTaggedCount = 0;
int32 AllHuntersDestroyed = 0; // flag if all hunters are destroyed
int32 MonsterDisabled = 0; // how many monsters have been subdued by Hunters

const float kDamageThreshold = 30.0f;
const float kMonsterKODamage = 50.0f;

struct MonsterStats
{
	int32 ScannedCount;
	int32 Tagged;
	int32 Aggressive;
	int32 Disabled;
};

MonsterStats MonsterMember[6] = {0}; // there are only 5 monsters, I'm going to use this 1 based

eGameStatus CheckWinCondition( tScript& tScript );

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kHunterTeam, 
	kMonsterTeam,
	kMonster5Team
};	
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kHydranMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kHunterTeam, kHydran);
	fMission.mSetTeamRace (kMonsterTeam, kMonster);
	fMission.mSetTeamRace (kMonster5Team, kMonster);

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations (kPlayerTeam, kHunterTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kMonsterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kMonster5Team, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kHunterTeam, kMonsterTeam, kTeamNeutral, kGoalCapture, 1.0, kTrue);
	fMission.mSetTeamRelations (kHunterTeam, kMonster5Team, kTeamNeutral, kGoalCapture, 1.0, kTrue);

	fMission.mSetTeamRelations (kMonsterTeam, kMonster5Team, kTeamAllied, kGoalCapture, 1.0, kTrue);

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* HunterTeam = fMission.mGetTeamHandle (kHunterTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* Monster5Team = fMission.mGetTeamHandle (kMonster5Team);

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.50f;
			}
			break;
		case 1:
			{
				TotalBPV *= 0.80f;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.20f;
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	// create Hunter
	HunterTeam->mCreateFleet (3, (3 * AveragePlayerBPV), kClassFrigate, kClassLightCruiser, 50, kStartPosition_S);

	// create Monster
	MonsterTeam->mCreateShip ( "M-AMS", kStartPosition_V);
	MonsterTeam->mCreateShip ( "M-AMS", kStartPosition_W);
	MonsterTeam->mCreateShip ( "M-AMS", kStartPosition_X);
	MonsterTeam->mCreateShip ( "M-AMS", kStartPosition_Y);

	// create Monster
	MonsterTeam->mCreateShip ( "M-AML", kStartPosition_Z);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* HunterTeam = fMission.mGetTeamHandle (kHunterTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* Monster5Team = fMission.mGetTeamHandle (kMonster5Team);

	tShipInfo* Ship;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents(kNotifyTransportedItemTo);
		Ship->mAccessSupplies( kSetTransporterItemCount, kTransBlackBox, 1 );
	}

	// configure Hunter
	for (Ship = HunterTeam->mGetFirstShip(); Ship != NULL; Ship = HunterTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);

		if( fMission.mGetDifficultyLevel() == 0 )
		{
			Ship->mChangeAIOfficerState( kFighterOfficer, kFalse );
		}
	}

	// configure Monster
	for (Ship = MonsterTeam->mGetFirstShip(); Ship != NULL; Ship = MonsterTeam->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
		Ship->mChangeName( kScovion_name );
	}

	// configure Monster5
	for (Ship = Monster5Team->mGetFirstShip(); Ship != NULL; Ship = Monster5Team->mGetNextShip())
	{
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mSetCommButton(1, kButtonHail_text);
		Ship->mChangeName( kScovion_name );
	}

	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer
	fMission.mSetTimedInterval (kTimerLimitSet, 1);

	// Timer to check distance from any of the monsters?
	// what about the other ships getting too close?
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* HunterTeam = fMission.mGetTeamHandle (kHunterTeam);
	tTeamInfo* MonsterTeam = fMission.mGetTeamHandle (kMonsterTeam);
	tTeamInfo* Monster5Team = fMission.mGetTeamHandle (kMonster5Team);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyTransportedItemTo:
			{
				tShipInfo* TransportTarget = fMission.mFindShipID( fEventID );

				if ( TransportTarget != NULL )
				{
					if (( TransportTarget->mGetTeam() == kMonsterTeam )
						&& ( fEventItem1 == kTransBlackBox ))
					{
						int32 count = 0;

						for ( tShipInfo* tempShip = MonsterTeam->mGetFirstShip(); tempShip != NULL; tempShip = MonsterTeam->mGetNextShip())
						{
							count++;

							if ( TransportTarget == tempShip )
							{
								if ( MonsterMember[count].Tagged == 0 )
								{
									MonsterMember[count].Tagged = 1;
									fMission.mDisplayMessage (kEventTag_msg, kScienceScriptMessage);
									MonsterTaggedCount++;
									break;
								}
								else 
								{
									fMission.mDisplayMessage (kEventAlreadyTagged_msg, kScienceScriptMessage);
									break;
								}
							}
						}

						if (MonsterTaggedCount == 5)
						{
							fMission.mDisplayMessage (kEventTagAll_msg, kScienceScriptMessage);

							fMission.mDisplayMessage( kReceivingTransmission_msg, kMissionScriptMessage );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 3 );

						}
						else
						{
							// add a bio monitor (black box) back to the player's tranporter room
							Ship->mAccessSupplies( kSetTransporterItemCount, kTransBlackBox, 1 );
						}
					}
					else
					{
						// LNF item is not black box, or the monster team
					}

					Ship->mSetShipCallbackEvents(kNotifyTransportedItemTo);
				}
			}
			break;

		case kNotifyBeginTractor:
			break;

		case kNotifyProbeLaunch:
			break;

		case kNotifyDeepScan:
			if ( fIsDeepScanOn == kTrue )
			{
				if ( PlayersTarget != NULL )
				{
					if ( fPlayerShip->mGetTacticalDistance( PlayersTarget ) <= 15.0 )
					{
						int32 count = 0;

						for ( tShipInfo* tempShip = MonsterTeam->mGetFirstShip(); tempShip != NULL; tempShip = MonsterTeam->mGetNextShip())
						{
							count++;

							if ( PlayersTarget == tempShip )
							{
								MonsterMember[count].ScannedCount++;

								if (( MonsterMember[count].ScannedCount == 3 )
									&& ( MonsterMember[count].Disabled != 1 ))
								{
									MonsterMember[count].Aggressive = 1;
									MonsterTeam->mGetThisShip(count)->mSetShipTarget( fPlayerShip, kVeryHighPriority );
								}
								break;
							}
						}
/*
						if(PlayersTarget == MonsterTeam->mGetThisShip(1))
						{
							MonsterMember[1].ScannedCount++;

							if ( MonsterMember[1].ScannedCount == 3 )
							{
								MonsterMember[1].Aggressive = 1;
								MonsterTeam->mGetThisShip(1)->mSetShipTarget( fPlayerShip, kVeryHighPriority );

								return kGameInProgress;
							}
						}
						else if(PlayersTarget == MonsterTeam->mGetThisShip(2))
						{
							MonsterMember[2].ScannedCount++;

							if ( MonsterMember[2].ScannedCount == 3 )
							{
								MonsterMember[2].Aggressive = 1;
								MonsterTeam->mGetThisShip(2)->mSetShipTarget( fPlayerShip, kVeryHighPriority );

								return kGameInProgress;
							}
						}
						else if(PlayersTarget == MonsterTeam->mGetThisShip(3))
						{
							MonsterMember[3].ScannedCount++;

							if ( MonsterMember[3].ScannedCount == 3 )
							{
								MonsterMember[3].Aggressive = 1;
								MonsterTeam->mGetThisShip(3)->mSetShipTarget( fPlayerShip, kVeryHighPriority );

								return kGameInProgress;
							}
						}
						else if(PlayersTarget == MonsterTeam->mGetThisShip(4))
						{
							MonsterMember[4].ScannedCount++;

							if ( MonsterMember[4].ScannedCount == 3 )
							{
								MonsterMember[4].Aggressive = 1;
								MonsterTeam->mGetThisShip(4)->mSetShipTarget( fPlayerShip, kVeryHighPriority );

								return kGameInProgress;
							}
						}
						else if(PlayersTarget == MonsterTeam->mGetThisShip(5))
						{
							MonsterMember[5].ScannedCount++;

							if ( MonsterMember[5].ScannedCount == 3 )
							{
								MonsterMember[5].Aggressive = 1;
								MonsterTeam->mGetThisShip(5)->mSetShipTarget( fPlayerShip, kVeryHighPriority );

								return kGameInProgress;
							}
						}
*/
					}

					fMission.mDisplayMessage( kEventScan_msg, kScienceScriptMessage );
				}
			}
			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID( fEventID );

				if ( HostileShip != NULL )
				{
					if ( HostileShip->mGetTeam() == kPlayerTeam )
					{
						int32 count = 0;

						for ( tShipInfo* tempShip = MonsterTeam->mGetFirstShip(); tempShip != NULL; tempShip = MonsterTeam->mGetNextShip())
						{
							count++;

							if ( Ship == tempShip )
							{
								if ( MonsterMember[count].Disabled != 1 )
								{
									MonsterMember[count].Aggressive = 1;
									MonsterTeam->mGetThisShip(count)->mSetShipTarget( HostileShip, kVeryHighPriority );
								}
								break;
							}
						}
					}
					else if ( HostileShip->mGetTeam() == kHunterTeam )
					{
						// if the Hunter team damages a monster >= kMonsterKODamage
						// then set a disabled flag on that monster, 
						// increment a disabled counter
						// remove the monster as a target for the Hunters
						// and check if all the monsters have been disabled, if so, playerloses

						if (( Ship->mGetTeam() == kMonsterTeam )
							|| ( Ship->mGetTeam() == kMonster5Team ))
						{
							if ( Ship->mGetDamagePercent() >= kMonsterKODamage )
							{
								MonsterDisabled++;

								if ( MonsterDisabled == 1 )
								{
									fMission.mDisplayMessage( kMonsterDisabled_msg, kScienceScriptMessage );
								}
								else if ( MonsterDisabled == 2 )
								{
									fMission.mDisplayMessage( kAnotherMonsterDisabled_msg, kScienceScriptMessage );
								}

								int32 count = 0;

								for ( tShipInfo* tempShip = MonsterTeam->mGetFirstShip(); tempShip != NULL; tempShip = MonsterTeam->mGetNextShip())
								{
									count++;

									if ( Ship == tempShip )
									{
										MonsterMember[count].Disabled = 1;
										break;
									}
								}

								MonsterTeam->mRemoveShipAsTarget( Ship );
								Monster5Team->mRemoveShipAsTarget( Ship );
								Ship->mChangeName( kMonsterDisabled_name );
								Ship->mSetShipAIControlled( kFalse );
								Ship->mSetShipThrust( 0 );

								if ( MonsterDisabled == 5 )
								{
									return kPlayerLoses;
								}
								else
								{
									return kGameInProgress;
								}
							}
							else
							{
								Ship->mSetShipCallbackEvents( kNotifyDamage );
							}
						}
					}

					Ship->mSetShipCallbackEvents( kNotifyDamage );
				}
			}
			break;

		case kNotifyAtDistance:
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// Set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kHunterTeam, kTeamHostile, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kPlayerTeam, kMonsterTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kMonster5Team, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kHunterTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kHunterTeam, kMonsterTeam, kTeamNeutral, kGoalCripple, 1.0);//***
					fMission.mSetTeamRelations (kHunterTeam, kMonster5Team, kTeamNeutral, kGoalCripple, 1.0);//***

					fMission.mSetTeamRelations (kMonsterTeam, kHunterTeam, kTeamHostile, kGoalDestroy, 1.0);//***
					fMission.mSetTeamRelations (kMonsterTeam, kMonster5Team, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kMonster5Team, kHunterTeam, kTeamHostile, kGoalDestroy, 1.0);//***
					break;

				case kTimerLimitSet_2:
					// LNF need a timer to check the distance to a monster and if the player's shields are up
					// fMission.mDisplayMessage( kEventShield_msg, kScienceScriptMessage );
					break;

				case kTimerLimitSet_3:
					// play message when done planting all objects
					if ( TalkedWithNumber5 != 1 )
					{
						fMission.mDisplayMessage(kMonsterHailTag_1_msg, kMissionScriptMessage);
						fMission.mDisplayMessage(kMonsterHailTag_2_msg, kMissionScriptMessage);
						fMission.mDisplayMessage(kMonsterHailTag_3_msg, kMissionScriptMessage);
						fMission.mDisplayMessage(kMonsterHailTag_4_msg, kMissionScriptMessage);
						TalkedWithNumber5 = 1;
					}

					fMission.mSetTimedInterval( kTimerLimitSet_4, 7 );

					break;

				case kTimerLimitSet_4:
					// check the win condition

					return CheckWinCondition( *this );

					break;

				case kTimerTurnsSet:
					break;

				case kTimerTurnsSet_8:
					// set the Monster5 comm buttons back to HAIL
					for (Ship = Monster5Team->mGetFirstShip(); Ship != NULL; Ship = Monster5Team->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_9:
					// set the Monster comm buttons back to HAIL
					for (Ship = MonsterTeam->mGetFirstShip(); Ship != NULL; Ship = MonsterTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Hunter comm buttons back to HAIL
					for (Ship = HunterTeam->mGetFirstShip(); Ship != NULL; Ship = HunterTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;
		
		case kNotifyAtDeath:
			if (Ship->mGetOriginalTeam() == kHunterTeam)
			{
				HunterDestroyed++;

				if ( HunterTeam->mGetShipsInPlayCount() == 0 )
				{
					AllHuntersDestroyed = 1;

					// check win condition
					fMission.mSetTimedInterval( kTimerLimitSet_4, 7 );
				}
			}
			else if (Ship->mGetOriginalTeam() == kMonsterTeam)
			{
				MonsterDestroyed++;

				if (MonsterTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetOriginalTeam() == kMonster5Team)
			{
				MonsterDestroyed++;

				if (Monster5Team->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetOriginalTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;

				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}

			break;

		case kNotifyOnCapture:

			fMission.mAddToBonusPrestige( kCaptureBonus );

			if (Ship->mGetOriginalTeam() == kHunterTeam)
			{
				HunterDestroyed++;

				if ( HunterTeam->mGetShipsInPlayCount() == 0 )
				{
					AllHuntersDestroyed = 1;

					// check win condition
					fMission.mSetTimedInterval( kTimerLimitSet_4, 7 );
				}
			}
			else if (Ship->mGetOriginalTeam() == kMonsterTeam)
			{
				MonsterDestroyed++;

				if (MonsterTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetOriginalTeam() == kMonster5Team)
			{
				MonsterDestroyed++;

				if (Monster5Team->mGetShipsInPlayCount() == 0)
				{
				}
			}
			break;

		case kNotifyDisengaged:
			if (Ship->mGetOriginalTeam() == kHunterTeam)
			{
				HunterDestroyed++;

				if ( HunterTeam->mGetShipsInPlayCount() == 0 )
				{
					AllHuntersDestroyed = 1;

					// check win condition
					fMission.mSetTimedInterval( kTimerLimitSet_4, 7 );
				}
			}
			else if (Ship->mGetOriginalTeam() == kMonsterTeam)
			{
				MonsterDestroyed++;

				if (MonsterTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetOriginalTeam() == kMonster5Team)
			{
				MonsterDestroyed++;

				if (Monster5Team->mGetShipsInPlayCount() == 0)
				{
				}
			}
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kHunterTeam:
								if (AtWarWithHunter == 0)
								{
									fMission.mDisplayMessage(kHunterHailNic_msg, kMissionScriptMessage);
								}
								else 
								{
									fMission.mDisplayMessage(kHunterHailCombat_msg, kMissionScriptMessage);
								}
								// LNF add betrayal message
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kMonsterTeam:
								if ( Ship == MonsterTeam->mGetThisShip( 5 ) )
								{
									if ((MonsterMember[5].Aggressive == 0) && (MonsterMember[5].Tagged == 0) && (TalkedWithNumber5 == 0))
									{
										fMission.mDisplayMessage(kMonsterHailNoTag_1_msg, kMissionScriptMessage);
										fMission.mDisplayMessage(kMonsterHailNoTag_2_msg, kMissionScriptMessage);
										fMission.mDisplayMessage(kMonsterHailNoTag_3_msg, kMissionScriptMessage);
										fMission.mDisplayMessage(kMonsterHailNoTag_4_msg, kMissionScriptMessage);
										TalkedWithNumber5 = 1;
									}
									else if ((MonsterMember[5].Aggressive == 0) && (MonsterMember[5].Tagged == 1) && (TalkedWithNumber5 == 0))
									{
										fMission.mDisplayMessage(kMonsterHailTag_1_msg, kMissionScriptMessage);
										fMission.mDisplayMessage(kMonsterHailTag_2_msg, kMissionScriptMessage);
										fMission.mDisplayMessage(kMonsterHailTag_3_msg, kMissionScriptMessage);
										fMission.mDisplayMessage(kMonsterHailTag_4_msg, kMissionScriptMessage);
										TalkedWithNumber5 = 1;
									}
									else if ((MonsterMember[5].Aggressive == 0) && (TalkedWithNumber5 == 1))
									{
										fMission.mDisplayMessage(kMonsterHailTalk_1_msg, kMissionScriptMessage);
										fMission.mDisplayMessage(kMonsterHailTalk_2_msg, kMissionScriptMessage);
										fMission.mDisplayMessage(kMonsterHailTalk_3_msg, kMissionScriptMessage);
										fMission.mDisplayMessage(kMonsterHailTalk_4_msg, kMissionScriptMessage);
									}
									else if (MonsterMember[5].Aggressive == 1)
									{
										fMission.mDisplayMessage(kMonsterHailAggressive_msg, kMissionScriptMessage);
									}

									Ship->mSetCommButton (1, kButtonTaunt_text);
									Ship->mSetCommButton (2, kButtonSurrender_text);

									// check win condition
									fMission.mSetTimedInterval( kTimerLimitSet_4, 7 );

								}
								else
								{
									fMission.mDisplayMessage(kMonsterHailMute_msg, kMissionScriptMessage);
								}

								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail and blank.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kMonster5Team:
								if ((MonsterMember[5].Aggressive == 0) && (MonsterMember[5].Tagged == 0) && (TalkedWithNumber5 == 0))
								{
									fMission.mDisplayMessage(kMonsterHailNoTag_1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailNoTag_2_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailNoTag_3_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailNoTag_4_msg, kMissionScriptMessage);
									TalkedWithNumber5 = 1;
								}
								else if ((MonsterMember[5].Aggressive == 0) && (MonsterMember[5].Tagged == 1) && (TalkedWithNumber5 == 0))
								{
									fMission.mDisplayMessage(kMonsterHailTag_1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailTag_2_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailTag_3_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailTag_4_msg, kMissionScriptMessage);
									TalkedWithNumber5 = 1;
								}
								else if ((MonsterMember[5].Aggressive == 0) && (TalkedWithNumber5 == 1))
								{
									fMission.mDisplayMessage(kMonsterHailTalk_1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailTalk_2_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailTalk_3_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kMonsterHailTalk_4_msg, kMissionScriptMessage);
								}
								else if (MonsterMember[5].Aggressive == 1)
								{
									fMission.mDisplayMessage(kMonsterHailAggressive_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);

								// check win condition
								fMission.mSetTimedInterval( kTimerLimitSet_4, 7 );

								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kHunterTeam:
								if (Ship->mGetDamagePercent() < kDamageThreshold)
								{
									fMission.mDisplayMessage(kHunterSurrenderOK_msg, kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage(kHunterSurrenderDamaged_msg, kMissionScriptMessage);
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kMonsterTeam:
								fMission.mDisplayMessage(kMonsterHailMute_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kMonster5Team:
								fMission.mDisplayMessage(kMonsterSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kHunterTeam:
								if (Ship->mGetDamagePercent() < kDamageThreshold)
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kHunterTauntOK_1_msg, kHunterTauntDamaged_3_msg), kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kHunterTauntDamaged_1_msg, kHunterTauntDamaged_3_msg), kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kMonsterTeam:
								fMission.mDisplayMessage(kMonsterHailMute_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;

							case kMonster5Team:
								fMission.mDisplayMessage(kMonsterSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_9, 2);
								break;
						}
						break;
				}
			}
			break;
	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;

	VictoryPoints = (MonsterTaggedCount * 16) - (MonsterDestroyed * 30) + (TalkedWithNumber5 * 20)
					- ((1 - AllHuntersDestroyed) * 100);
	// LNF check if the Hunter team is completely destroyed or disabled or Disengaged

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kVictory;
	}
	else if (VictoryPoints <= 49)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	tTeamInfo* HunterTeam = fMission.mGetTeamHandle (kHunterTeam);

	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

	if ( VictoryLevel >= kDefeat )
	{
		if ( HunterTeam->mGetShipsInPlayCount() > 0 )
		{
			fMission.mAddToDebriefingList( kDefeatHuntersRemain_msg );
		}
		else
		{
			fMission.mAddToDebriefingList( kDefeatNoHunters_msg );
		}
	}

	fMission.mAddToDebriefingList (kDismissedShort_msg);

/*
	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( SesOrbiting != 0 ) ? fMission.mAddToDebriefingList( kSesOrbitingTrue_msg ) : SuccessCount++;
	( Unprovoked == 0 ) ? fMission.mAddToDebriefingList( kUnprovokedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( SesOrbiting == 0 ) ? fMission.mAddToDebriefingList( kSesOrbitingFalse_msg ) : FailureCount++;
	( Unprovoked != 0 ) ? fMission.mAddToDebriefingList( kUnprovokedFalse_msg ) : FailureCount++;
	if ( FailureCount == 2 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}
*/

	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// check for the win condition
eGameStatus CheckWinCondition( tScript& tScript )
{
	if (( AllHuntersDestroyed == 1 )
		&& ( MonsterTaggedCount == 5 ))
	{
		return kPlayerWins;
	}
	else
	{
		return kGameInProgress;
	}
}

