#include "scriptinterface.h"

#include "Kli_2aMissionText.h"
#include "Kli_2aMissionMaps.h"

int32 LyranDestroyed = 0;
int32 PlayerDestroyed = 0;
int32 PlayerCaptured = 0;
int32 StartMission = 0;
int32 LyranHailed = 0;
int32 LyranWarning = 0;
int32 AtWarWithLyran = 0;
int32 LyranCaptured = 0;

const int32 kWormholeX = 9; // X coord of wormhole
const int32 kWormholeY = 13; // Y coord of wormhole
const float kWormholeWarningDistance = 30.0; // distance to warn player of wormhole
const float kWormholeDistance = 0.0; // distance to disengage player
const float kPatrolAttackRadius = 40.0;

bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key

void PatrolRoute(tShipInfo* Ship, int32 ShipNumber);

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kLyranTeam, 
	kPlanetTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kKlingMission;
	Specs->fEnemyRace = kLyranMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = kLyran;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kLyranTeam, kLyran);
	fMission.mSetTeamRace (kPlanetTeam, kLyran);

	// LNF check the relationships from Dynaverse first?
	// setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kLyranTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	
	fMission.mSetTeamRelations (kLyranTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	//return kFalse; // use the script given ship
	return kTrue; //kTrue uses player's own ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* LyranTeam = fMission.mGetTeamHandle (kLyranTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	// create Lyran team
	// identical ships unless we want to check the captured ship and use that same ship
	// in the next mission, part II
	LyranTeam->mCreateShip ("L-CL", kStartPosition_Y); // use this ship in the second part
	LyranTeam->mCreateShip ("L-CL", kStartPosition_Z); // use this ship in the second part

	// create planet
	PlanetTeam->mCreateShip ("PL1", kStartPosition_Specified, 25, 19);

}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* LyranTeam = fMission.mGetTeamHandle (kLyranTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);
	tShipInfo* Ship;

	int32 ShipNumber = 0; // ship number to set patrol route

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipThrust (5);
		Ship->mTrackTacticalDistance(kWormholeWarningDistance, kWormholeX, kWormholeY); // distance to worm hole
		//Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
		//Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
	}

	// LNF for balance, disable more weapons?  this one is weird since the Lyran fleet is
	// LNF fixed in class, and the player's fleet will change.  We can add ships, I suppose
	// LNF and if the player captures them, blow them up. although that may screw the player
	// LNF if they have wasted all their marines on capturing those ships.
	// LNF maybe these will just be easy missions.

	// configure Lyran ships
	for (Ship = LyranTeam->mGetFirstShip(); Ship != NULL; Ship = LyranTeam->mGetNextShip())
	{
		Ship->mSetCommButton ( 1, kButtonHail_text );
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		PatrolRoute( Ship, ShipNumber );

		Ship->mSetShipMaxSpeed( 12 );

		ShipNumber++;
		// LNF
		//Ship->mAccessSupplies( kSetSupplyCount, kSupplyMarines, 0 );
		//Ship->mAccessSupplies( kSetSupplyCount, kSupplySpareParts, 8 );
		//Ship->mDamageSystem(kDamageWeaponsAndSystems,50);
	}

	// configure planet


	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer to set relationships
	fMission.mSetTimedInterval (kTimerLimitSet, 3);

	// set the end game pause delay
	fMission.mSetTimedInterval (kTimerEndGamePause, 0);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* LyranTeam = fMission.mGetTeamHandle (kLyranTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	switch (Event)
	{
/*		case kNotifyTransportedItemTo:
			if (fEventItem1 == kTransSparePart)
			{
				fMission.mDisplayMessage( kLNFTransTo_msg, kMissionScriptMessage );
			}
			Ship->mSetShipCallbackEvents( kNotifyTransportedItemTo );
			break;

		case kNotifyTransportedItemFrom:
			fMission.mDisplayMessage( kLNFTransFrom_msg, kMissionScriptMessage );
			Ship->mSetShipCallbackEvents( kNotifyTransportedItemFrom );
			break;
*/
		case kNotifyDamage:
			if (AtWarWithLyran == 0)
			{
				if ( Ship->mGetTeam() == kLyranTeam )
				{
					fMission.mSetTeamRelations (kPlayerTeam, kLyranTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					Ship->mSetShipMaxSpeed( 31 );
					AtWarWithLyran = 1;
				}
			}
			break;

		case kNotifyOnCapture:

			fMission.mAddToBonusPrestige( kCaptureBonus );

			if ( Ship->mGetOriginalTeam() == kLyranTeam )
			{
				if ( LyranCaptured == 0 )
				{
					LyranCaptured = 1;
					fMission.mDisplayMessage(kEvent1_CapturedLyran_msg, kMissionScriptMessage);
					if ( LyranDestroyed == 0 )
					{
						fMission.mDisplayMessage( kEvent1_NeedToDestroyOther_msg, kMissionScriptMessage );
					}
					else
					{
						fMission.mDisplayMessage(kEvent1_CompletedMission_msg, kMissionScriptMessage);
						fMission.mSetGoalIndicator( kTrue, kWormholeX, kWormholeY );
					}
				}
				else
				{
					LyranCaptured = 2;
					fMission.mDisplayMessage(kEvent1_CompletedMission_msg, kMissionScriptMessage);
					fMission.mSetGoalIndicator( kTrue, kWormholeX, kWormholeY );
				}

			}
			else
			{
				if ( Ship != fPlayerShip )
				{
					PlayerCaptured++;
					fMission.mDisplayMessage( kPlayerTeamCaptured_msg, kMissionScriptMessage );
				}
			}

			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					if (StartMission == 0)
					{
						fMission.mSetTeamRelations( kPlayerTeam, kLyranTeam, kTeamNeutral, kGoalCapture, 1.0 );
						fMission.mSetTeamRelations( kPlayerTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

						fMission.mSetTeamRelations( kLyranTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0 );
						
						fMission.mSetTeamRelations (kLyranTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

						StartMission = 1;
						fMission.mDisplayMessage (kStartScenario_msg, kMissionScriptMessage);
						fMission.mSetTimedInterval (kTimerLimitSet, 3);
					}
					else if (((fPlayerShip->mGetTacticalDistance(LyranTeam->mGetThisShip(1)) < 85.0) ||
							(fPlayerShip->mGetTacticalDistance(LyranTeam->mGetThisShip(2)) < 85.0)) &&
							(LyranWarning == 0) && (LyranHailed == 0))
					{
						fMission.mDisplayMessage(kLyranWarn1_msg, kMissionScriptMessage);
						fMission.mDisplayMessage(kLyranWarn2_msg, kMissionScriptMessage);
						LyranWarning = 1;
						LyranTeam->mClearCommands();
						fMission.mSetTeamRelations( kPlayerTeam, kLyranTeam, kTeamAtWar, kGoalCapture, 1.0 );
						fMission.mSetTeamRelations( kLyranTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0 );
					}
					else
					{
						fMission.mSetTimedInterval (kTimerLimitSet, 3);
					}
					break;

				case kTimerTurnsSet_10:
					// set the Lyran comm buttons back to HAIL
					for (Ship = LyranTeam->mGetFirstShip(); Ship != NULL; Ship = LyranTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;

		case kNotifyAtDistance:
			if ( Ship == fPlayerShip )
			{
				if (fPlayerShip->mGetTrackingDistance() == kWormholeWarningDistance)
				{
					// warn player approaching wormhole
					fMission.mDisplayMessage(kWormholeWarning_msg, kScienceScriptMessage);
					fPlayerShip->mTrackTacticalDistance(kWormholeDistance, kWormholeX, kWormholeY);
				}
				else if (fPlayerShip->mGetTrackingDistance() == kWormholeDistance)
				{
					fMission.mDisplayMessage( kEnteringWormhole_msg, kMissionScriptMessage );
					fMission.mSetGoalIndicator( kFalse, 0, 0 );
					return kPlayerDisengaged;
				}
			}
			break;
		
		case kNotifyAtDeath:
			if ( Ship->mGetOriginalTeam() == kLyranTeam )
			{
				LyranDestroyed++;
				fMission.mDisplayMessage( kEvent6_KillingALyran_msg, kMissionScriptMessage, Ship );

				// if the ship that died was NOT captured
				if ( Ship->mGetTeam() == Ship->mGetOriginalTeam() )
				{
					if (LyranTeam->mGetShipsInPlayCount() == 0) // if it was the last ship
					{
						if ( LyranCaptured == 1 )
						{
							fMission.mDisplayMessage( kEvent1_CompletedMission_msg, kMissionScriptMessage );
							fMission.mSetGoalIndicator( kTrue, kWormholeX, kWormholeY);
						}
						else
						{
							fMission.mDisplayMessage( kEvent6_BothDestroyed_msg, kMissionScriptMessage );
							fMission.mSetGoalIndicator( kTrue, kWormholeX, kWormholeY);
							//return kPlayerLoses;
						}
					}
					else
					{
						fMission.mDisplayMessage( kEvent6_NeedToCaptureOther_msg, kMissionScriptMessage );
					}
				}
				else // the ship was captured
				{
					// LNF other is captured - player wins
					// LNF other is destroyed - player fails
					// LNF other is alive - tell player to capture other
					if ( LyranCaptured == 2 )
					{
						LyranCaptured = 1;
					}
					else if ( LyranDestroyed > 1 )
					{
						fMission.mDisplayMessage( kEvent6_BothDestroyed_msg, kMissionScriptMessage );
						fMission.mSetGoalIndicator( kTrue, kWormholeX, kWormholeY);
					}
					else
					{
						fMission.mDisplayMessage( kEvent6_NeedToCaptureOther_msg, kMissionScriptMessage );
					}
				}
			}
			else if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			else
			{
			}
			break;

		case kNotifyDisengaged:
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{

				switch (fEventID)
				{
					case kButtonHail_text:
						if (AtWarWithLyran == 0)
						{
							fMission.mDisplayMessage (kLyranHailEnemyNic_msg, kMissionScriptMessage);
							LyranTeam->mClearCommands();
							fMission.mSetTeamRelations( kPlayerTeam, kLyranTeam, kTeamAtWar, kGoalCapture, 1.0 );
							fMission.mSetTeamRelations( kLyranTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0 );
						}
						else
						{
							fMission.mDisplayMessage (kLyranHail_msg, kMissionScriptMessage);
						}
						LyranHailed = 1;

						Ship->mSetCommButton (1, kButtonTaunt_text);
						Ship->mSetCommButton (2, kButtonSurrender_text);
						// If the Taunt or Surrender buttons aren't used in X turns
						// return the buttons to Hail and blank.
						fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						break;

					case kButtonSurrender_text:
						if (Ship->mGetDamagePercent() < 50.0)
						{
							fMission.mDisplayMessage (kLyranSurrenderOK_msg, kMissionScriptMessage);
						}
						else
						{
							fMission.mDisplayMessage (kLyranSurrenderDamaged_msg, kMissionScriptMessage);
						}
						Ship->mSetCommButton (2, kButtonSurrender_text);
						fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						break;

					case kButtonTaunt_text:
						if (AtWarWithLyran == 0)
						{
							fMission.mDisplayMessage (kLyranTauntNic_msg, kMissionScriptMessage);
						}
						else
						{
							fMission.mDisplayMessage (fMission.mRandomInt32 (kLyranTaunt1_msg, kLyranTaunt3_msg), kMissionScriptMessage);
						}
						Ship->mSetCommButton (1, kButtonTaunt_text);
						fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
						break;

				}
			}
			break;
	}

	return fMission.mGetGameStatus();
	//return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;
	
	VictoryPoints = LyranCaptured * 100;
	
	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if (VictoryPoints <= 99)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}


	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	( LyranCaptured != 0 ) ? fMission.mAddToDebriefingList( kLyranCapturedTrue_msg ) : SuccessCount++;
	if ( SuccessCount == 1 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	fMission.mAddToDebriefingList( kFailedObjectives_msg );

	( LyranCaptured == 0 ) ? fMission.mAddToDebriefingList( kLyranCapturedFalse_msg ) : FailureCount++;
	if ( FailureCount == 1 )
	{
		fMission.mAddToDebriefingList( kNone_msg );
	}

	// Force the selection of the next mission
	// LNF unless the player dies.  Then what do I do?
	if ( VictoryLevel < kDefeat)
	{
		fMission.mScheduleMission(kNextMission_text);
	}

}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// set patrol for vessels
void PatrolRoute(tShipInfo* Ship, int32 ShipNumber)
{
	tMapPoint waypoints1[7]= // ship Y
	{
		21, 20,
		22, 16,
		25, 14,
		31, 16,
		33, 19,
		29, 21,
		25, 22
	};

	tMapPoint waypoints2[6]= // ship Z
	{
		14, 25,
		17, 28,
		22, 29,
		28, 26,
		25, 23,
		19, 22
	};

	tMapPoint PatrolPoints[3][10];

	switch (ShipNumber)
	{
		case 0:
			PatrolPoints[0][0] = waypoints1[0];
			PatrolPoints[0][1] = waypoints1[1];
			PatrolPoints[0][2] = waypoints1[2];
			PatrolPoints[0][3] = waypoints1[3];
			PatrolPoints[0][4] = waypoints1[4];
			PatrolPoints[0][5] = waypoints1[5];
			PatrolPoints[0][6] = waypoints1[6];
			Ship->mPatrol (kPatrolAttackRadius, 7, &PatrolPoints[0][0], kHighPriority);
			break;
		case 1:
			PatrolPoints[1][0] = waypoints2[0];
			PatrolPoints[1][1] = waypoints2[1];
			PatrolPoints[1][2] = waypoints2[2];
			PatrolPoints[1][3] = waypoints2[3];
			PatrolPoints[1][4] = waypoints2[4];
			PatrolPoints[1][5] = waypoints2[5];
			Ship->mPatrol (kPatrolAttackRadius, 6, &PatrolPoints[1][0], kHighPriority);
			break;
		}
}