#include "scriptinterface.h"

#include "Mul_FreeForAllText.h"
#include "Mul_FreeForAllMaps.h"

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,
};

//---------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------
//
#include "teamtracker.h"
tTeamTrackerClass fTeamTracker;

//---------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kMultiplayerMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kAllRaceMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;

	// multiplayer menu specific settings:
	//		max players (fWait)
	//		max ships (fNumShips)
	//		enable supply dock
	//		disable teams
	//		disable AI
	Specs->fNumShips = MissSpecAllShips;			// max number of ships per player
	Specs->fWait = 6;								// max number of players
	Specs->fEnableSupplyDock = kEnableSupplyDock;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	// must setup the races and relations of the reserved teams in the script
	fMission.mSetMultiplayerTeams (info);

	// setup the team tracker class
	fTeamTracker.mInitialize (kPlayerTeam, info, 0);

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	int32 position = fMission.mGetTeamHandle (kPlayerTeam)->mGetMultiplayerTeam() + kStartPosition_G;
	Ship.fShipPosition = position | kGroupAround;
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	// only the host needs to create the AI ships
	if (fThisScriptIsTheHost)
	{
		tTeamInfo* Team;
		int32 position;

		// create friendly AI ships first
		for (int32 i = 0; i < fMission.fTotalAlliedCPUTeams; i++)
		{
			Team = fMission.mGetTeamHandle (kCPU_AlliedTeams, i);
			position = Team->mGetMultiplayerTeam() + kStartPosition_G;
			Team->mCreateCPUFleet (20, position | kGroupAround);
		}

		// create enemy AI ships last
		for (i = 0; i < fMission.fTotalEnemyCPUTeams; i++)
		{
			Team = fMission.mGetTeamHandle (kCPU_EnemyTeams, i);
			position = Team->mGetMultiplayerTeam() + kStartPosition_G;
			Team->mCreateCPUFleet (20, position | kGroupAround);
		}
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	// only the host needs to setup the AI ships
	if (fThisScriptIsTheHost)
	{
	}
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	// game ends for everyone when the last man is standing
	switch (Event)
	{
		case kNotifyAtDeath:
			{
				fTeamTracker.mShipDied (Ship->mGetTeam());
				if (Ship->mGetTeam() == kPlayerTeam
					&& fMission.mGetTeamHandle (kPlayerTeam)->mGetShipsInPlayCount() == 0)
				{
					fMission.mDisplayMessage( kYouAreDead_msg );
				}
			}
			break;

		case kNotifyHostChanged:
			fThisScriptIsTheHost = kTrue;
			break;

		case kNotifyDisengaged:
			if (Ship == fPlayerShip)
			{
				fMission.mDisplayMessage( kYouDisengaged_msg );
				fMission.mGetTeamHandle (kPlayerTeam)->mDisengageShips (kByMapEdge, kAbsolutePriority);
				return kPlayerLoses;
			}
			fTeamTracker.mShipDied (Ship->mGetTeam());
			break;

		case kNotifyPlayerLeaving:
			fTeamTracker.mTeamGone (eTeamID(fEventID), NULL);
			break;
	}
	return fTeamTracker.mGetTeamsStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	if (fTeamTracker.mGetTeamsStatus() == kPlayerWins)
	{
		return kAstoundingVictory;
	}

	return kDevastatingDefeat;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList( kYouScored_msg );
	fMission.mAddToDebriefingList( VictoryLevel + kAstoundingVictory_msg );
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}


