#include "scriptinterface.h"

#include "MissionText.h"
#include "MissionMaps.h"

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
};

int32	TotalGamesWon;
int32	TotalGamesLost;
tString PlayersShipsClassName[3];
int32	RespawningLocalShips;
iTruth	ControlFirstShipRespawned;
int32	TotalShipsOnTeam;

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kMultiplayerMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kAllRaceMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;

	// multiplayer menu specific settings:
	//		max players (fWait)
	//		max ships (fNumShips)
	//		enable supply dock
	//		disable teams
	//		disable AI
	Specs->fNumShips = MissSpecAllShips;			// max number of ships per player
	Specs->fWait = 2;								// max number of players
	Specs->fEnableSupplyDock = kEnableSupplyDock | kDisableAllTeams | kDisableAI;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	TotalGamesWon = 0;
	TotalGamesLost = 0;
	RespawningLocalShips = 0;

	fMission.mSetPrestigePoints (0, 0, 0, 0, 0);

	// must setup the races and relations of the reserved teams in the script
	fMission.mSetMultiplayerTeams (info);

	// setup the races of the non-reserved teams in the script

	// setup team relations and target goals of non-reserved teams

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// players on team 1 will use start positions G-L, team 2 uses U-Z
	if (Ship.fTeam == kTeam2)
	{
		Ship.fShipPosition += kStartPosition_U - kStartPosition_G;
	}
	Ship.fShipPosition |= kGroupAround;
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tShipInfo* Ship;

	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	TotalShipsOnTeam = 0;
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		// save for respawn
		PlayersShipsClassName[TotalShipsOnTeam++]= Ship->mGetClassName();
	}

	fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	if (RespawningLocalShips && Ship->mGetTeam() == kPlayerTeam)
	{
		// reset all systems of the ship - this will update the ship to its original state
		Ship->mDamageSystem (kDamageWeaponsSystemsAndShields, 0);
		Ship->mAccessSupplies (kResetAllItems, 0, 0);
		Ship->mSetTargetable( kFalse );
		fMission.mSetTimedInterval( kTimerTurnsSet, 1 );

		RespawningLocalShips--;
		if (ControlFirstShipRespawned)
		{
			Ship->mSetShipPlayerControlled();
			ControlFirstShipRespawned = kFalse;
		}
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* Team;

	switch (Event)
	{
		case kNotifyAtDeath:
			{
				eTeamID ship_team = Ship->mGetTeam();
				Team = fMission.mGetTeamHandle (ship_team);
				if (Team->mGetShipsInPlayCount() > 0)
				{
					break;		// leave if team has more ships in play
				}
				eTimerIntervalTypes timer_callback;
				if (ship_team == kPlayerTeam)
				{
					fMission.mDisplayMessage (kPlayerIsDestroyed_msg, kNonCommScriptMessage);
					TotalGamesLost++;
					if (TotalGamesLost == 2)
					{
						return kPlayerLoses;
					}
					timer_callback = kTimerLimitSet;
				}
				else
				{
					fMission.mDisplayMessage (kOpponentDestroyed_msg, kNonCommScriptMessage);
					TotalGamesWon++;
					if (TotalGamesWon == 2)
					{
						return kPlayerWins;
					}
					timer_callback = kTimerLimitSet_2;
				}
				// set up 8 second callback to reset ships
				fMission.mSetTimedInterval (timer_callback, 8);

				// display starting round 2 or round 3 messages
				if (TotalGamesWon + TotalGamesLost == 1)
				{
					fMission.mDisplayMessage (kStartRound2_msg, kNonCommScriptMessage);
				}
				else
				{
					fMission.mDisplayMessage (kStartRound3_msg, kNonCommScriptMessage);
				}
			}
			break;

		case kNotifyAtTimedInterval:
			{
				if( fEventID == kTimerTurnsSet )
				{
					for( Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip() )
					{
						Ship->mSetTargetable( kTrue );
					}
					break;
				}

				RespawningLocalShips = 0;
				Team = fMission.mGetTeamHandle (kPlayerTeam);
				if (fEventID == kTimerLimitSet)
				{
					// this scripts player lost his ships last round - create a new fleet
					ControlFirstShipRespawned = kTrue;
					for (int32 i = 0; i < TotalShipsOnTeam; i++)
					{
						Ship = Team->mGetThisShip (i + 1);
						RespawningLocalShips++;
						Team->mCreateShip (i, PlayersShipsClassName[i].mCharStr(), Ship->mGetStartPositionIndex(),
										   Ship->mGetStartX(), Ship->mGetStartY(), Ship->mGetEndX(), Ship->mGetEndY());
					}
					fMission.mDisplayMessage( kPlayerIsBack_msg, kNonCommScriptMessage );
					if( TotalGamesLost == 1 )
					{
						fMission.mDisplayMessage( kOneLife_msg, kNonCommScriptMessage );
					}
				}
				else	// this scripts player won last round - respawn all ships
				{
					int32 base = 1;
					if (TotalGamesLost == 1)
					{
						base += TotalShipsOnTeam;
					}
					ControlFirstShipRespawned = kFalse;
					for (int32 i = 0; i < TotalShipsOnTeam; i++)
					{
						Ship = Team->mGetThisShip (i + base);
						if (Ship->mTestShipStatus (kShipDestroyed))
						{
							RespawningLocalShips++;
							Team->mCreateShip (i, PlayersShipsClassName[i].mCharStr(), Ship->mGetStartPositionIndex(),
											   Ship->mGetStartX(), Ship->mGetStartY(), Ship->mGetEndX(), Ship->mGetEndY());
						}
						else
						{
							// reset all systems of any ships still in play
							Ship->mDamageSystem (kDamageWeaponsSystemsAndShields, 0);
							Ship->mAccessSupplies (kResetAllItems, 0, 0);
						}
					}
					fMission.mDisplayMessage( kOpponentBack_msg, kNonCommScriptMessage );
					if( TotalGamesLost == 1 )
					{
						fMission.mDisplayMessage( kOneLife_msg, kNonCommScriptMessage );
					}
				}
			}
			break;

		case kNotifyDisengaged:
			fEventID = Ship->mGetTeam();
			// case falls thru
		case kNotifyPlayerLeaving:
			if (fEventID != kPlayerTeam)
			{
				TotalGamesWon = 2;
				return kPlayerWins;
			}
			break;
	}
	return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	if (TotalGamesWon == 2)
	{
		return kAstoundingVictory;
	}
	return kDevastatingDefeat;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if (VictoryLevel == kAstoundingVictory)
	{
		fMission.mAddToDebriefingList (kWinner_msg);
	}
	else
	{
		fMission.mAddToDebriefingList (kLoser_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
