#include "scriptinterface.h"

#include "Rom_3MissionText.h"
#include "Rom_3MissionMaps.h"

//***      balance      ***
eClassTypes PlayerShipClass = kClassShuttle;
eClassTypes LargestEnemyShipClass = kClassFrigate;
eClassTypes NewEnemyShipClass( tScript& tScript, eClassTypes LargestEnemyShipClass );
//***                   ***

int32 PlayerDestroyed = 0;
int32 PlagueDestroyed = 0;
int32 QShipDestroyed = 0;
int32 EscortDestroyed = 0;
int32 AllQShipsDestroyed = 0;
int32 AllEscortsDestroyed = 0;
int32 AllPlagueShipsDestroyed = 0;
int32 Surrender = 0;
int32 Agreement = 0;
int32 AtWarWithPlague = 0;
int32 Elite = 0;
int32 EscapedShips = 0;
int32 GoalIndicator = 0;
int32 PlagueShipEscaped = 0;

bool FoundClosestShip = false;
bool PlagueShipsStopped = false;
bool PlayerUsedESCKey = false; // if the player Ends Mission with the ESC key
bool AllEnemyShipsDestroyed = false; // player kills all plague ships

const float kEscortTeamSurrenderThreshold = 60.0;
const float kDistanceToPlagueShips = 100.0;

void OrbitPlanet(tTeamInfo* OrbitTeam);
eGameStatus CheckWinCondition( tScript& tScript );

enum eEventShipStatus
{
	kEventShipDestroyed = 0,
	kEventShipCaptured,
	kEventShipDisengaged
};

void CheckTeamDestruction(tScript& tScript, tTeamInfo* EventTeam, eEventShipStatus EventShipStatus,
						  tShipInfo* Ship, tShipInfo* CapturingShip = NULL);

enum eEliteOrganizations
{
	kPraiex = 1,
	kShiar,
};

enum eGoalIndicatorState
{
	kOff = 0,
	kOn,
};

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kPlagueTeam, 
	kQShipTeam,
	kEscortTeam,
	kPlanetTeam
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kRomulanMission;
	Specs->fEnemyRace = kAllRaceMission; //LNF will Dynaverse take care of this?
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = kUniqueMission;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (200, 160, 100, 40, 0);

	// must set up the races of the teams in the script
	info.fEnemyRace = info.fPlayerRace;
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kPlagueTeam, kRomulan);
	fMission.mSetTeamRace (kQShipTeam, kRomulan);
	fMission.mSetTeamRace (kEscortTeam, kRomulan);
	fMission.mSetTeamRace (kPlanetTeam, kNeutralRace);

	// Set team relations to show up on Tactical map
	fMission.mSetTeamRelations (kPlayerTeam, kPlagueTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kQShipTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kEscortTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kPlagueTeam, kQShipTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlagueTeam, kEscortTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlagueTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kQShipTeam, kEscortTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kQShipTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kEscortTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

	if (fMission.mGetEliteOrganization() == kPraiex)
	{
		Elite = kPraiex;
	}
	else
	{
		Elite = kShiar;
	}

	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;  // use player's ship
	//return kFalse;	// use a given ship
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* PlagueTeam = fMission.mGetTeamHandle (kPlagueTeam);
	tTeamInfo* QShipTeam = fMission.mGetTeamHandle (kQShipTeam);
	tTeamInfo* EscortTeam = fMission.mGetTeamHandle (kEscortTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship = NULL;

	// Get the total BPV of the player and modify by a low and high percentage
	int32 LowBPV = PlayerTeam->mGetCombatBPV() * .80;
	int32 HighBPV = PlayerTeam->mGetCombatBPV() * 1.20;
	int32 TotalBPV = fMission.mRandomInt32 (LowBPV, HighBPV);

	// balance
	int32 NumberOfPlayersShips = PlayerTeam->mGetShipsInPlayCount();
	int32 DifLev = fMission.mGetDifficultyLevel();

	// determine the biggest ship class the player has
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		if (PlayerShipClass < Ship->mGetClassType())
		{
			PlayerShipClass = Ship->mGetClassType();
		}
	}

	switch ( DifLev )
	{
		case 0:
			{
				TotalBPV *= 0.60f;

				// set LargestEnemyShipClass to one SMALLER than the player
				if ( PlayerShipClass > kClassFrigate )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass - 1);
				}
			}
			break;
		case 1:
			{
				// do nothing
				LargestEnemyShipClass = PlayerShipClass;
			}
			break;
		case 2:
			{
				TotalBPV *= 1.5f;

				// set LargestEnemyShipClass to one LARGER than the player
				if ( PlayerShipClass < kClassBattleship )
				{
					LargestEnemyShipClass = (eClassTypes) ((int32)PlayerShipClass + 1);
				}
			}
			break;
	}

	int32 AveragePlayerBPV = TotalBPV / NumberOfPlayersShips;

	// create Plague freighters
	// it doesn't really matter what type since the weaps will be destroyed
	PlagueTeam->mCreateShip ("R-FSPh", kStartPosition_S);
	//PlagueTeam->mCreateShip ("R-FSPL", kStartPosition_T);
	PlagueTeam->mCreateShip ("R-FLPL", kStartPosition_U);
	//PlagueTeam->mCreateShip ("R-FSPh", kStartPosition_V);

	// create QShip team
	QShipTeam->mCreateShip ("R-FQS", kStartPosition_W);
	//QShipTeam->mCreateShip ("R-FQS", kStartPosition_X);
	QShipTeam->mCreateShip ("R-FQS", kStartPosition_Y);

	// create Escort team
	//EscortTeam->mCreateFleet (EnemyCount, (EnemyCount * (TotalBPV / EnemyCount)), kClassFrigate, kClassLightCruiser, 50, kStartPosition_Specified, 3, 23);
	EscortTeam->mCreateFleet (1, AveragePlayerBPV, kClassFrigate, LargestEnemyShipClass, 50, kStartPosition_Specified, 3, 23);

	// create Planet
	PlanetTeam->mCreateShip( "PL1", kStartPosition_Specified, 30, 10 );
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	if (Elite == kPraiex)
	{
		return (kPraiexBriefing_msg);
	}
	else
	{
		return (kShiarBriefing_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* PlagueTeam = fMission.mGetTeamHandle (kPlagueTeam);
	tTeamInfo* QShipTeam = fMission.mGetTeamHandle (kQShipTeam);
	tTeamInfo* EscortTeam = fMission.mGetTeamHandle (kEscortTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* Ship = NULL;
	int32 count = 0;

	// configure Player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
/*	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
	}
*/
	// configure Plague Freighters
	for (Ship = PlagueTeam->mGetFirstShip(); Ship != NULL; Ship = PlagueTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		Ship->mSetShipMaxSpeed (10);
		Ship->mSetFinalDestination();
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mDamageSystem(kDamageMaxWeapons, 100);

		Ship->mChangeName( kShip_name );
	}

	// configure QShip team
	count = 1;
	for (Ship = QShipTeam->mGetFirstShip(); Ship != NULL; Ship = QShipTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		//Ship->mSetShipMaxSpeed (10);
		//Ship->mSetFinalDestination();
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mDefendEntity( PlagueTeam->mGetThisShip( fMission.mRandomInt32(1, 2) ), 15.0, 15.0, kVeryHighPriority );

		Ship->mChangeName( kShip_name );
	}

	// configure Escort team
	for (Ship = EscortTeam->mGetFirstShip(); Ship != NULL; Ship = EscortTeam->mGetNextShip())
	{
		Ship->mSetCommButton (1, kButtonHail_text);
		//Ship->mSetShipMaxSpeed (10);
		//Ship->mSetFinalDestination();
		Ship->mSetShipCallbackEvents (kNotifyDamage);
		Ship->mDefendEntity( PlagueTeam->mGetThisShip( fMission.mRandomInt32(1, 2) ), 25.0, 30.0, kVeryHighPriority );

		Ship->mChangeName( kShip_name );
}

	// configure Planet
	PlanetTeam->mGetFirstShip()->mChangeName( kImmelask_name );
	
	fMission.mDisplayMessage (kStarting_msg, kNonCommScriptMessage);

	// Start a timer to reset the relationships
	fMission.mSetTimedInterval (kTimerLimitSet, 3);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* PlagueTeam = fMission.mGetTeamHandle (kPlagueTeam);
	tTeamInfo* QShipTeam = fMission.mGetTeamHandle (kQShipTeam);
	tTeamInfo* EscortTeam = fMission.mGetTeamHandle (kEscortTeam);
	tTeamInfo* PlanetTeam = fMission.mGetTeamHandle (kPlanetTeam);

	tShipInfo* PlayersTarget = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyAtDestination:
			PlagueTeam->mDisengageShips(kByMapEdge, kHighPriority);
			QShipTeam->mDisengageShips(kByMapEdge, kHighPriority);
			EscortTeam->mDisengageShips(kByMapEdge, kHighPriority);

			if ( Ship->mGetTeam() == kPlagueTeam )
			{
				// player loses if plague ships get here
				fMission.mDisplayMessage( kPlagueShipEscapes_msg, kMissionScriptMessage );
				PlagueShipEscaped = 1;
				return kPlayerLoses;
			}
			break;

		case kNotifyDamage:
			{
				tShipInfo* HostileShip = fMission.mFindShipID( fEventID );

				if ( HostileShip != NULL )
				{
					if (( HostileShip->mGetTeam() == kPlayerTeam ) && 
						(( Ship->mGetTeam() == kPlagueTeam ) || ( Ship->mGetTeam() == kQShipTeam ) ||
						( Ship->mGetTeam() == kEscortTeam )))
					{
						fMission.mSetTeamRelations (kPlayerTeam, kPlagueTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
						fMission.mSetTeamRelations (kPlayerTeam, kQShipTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
						fMission.mSetTeamRelations (kPlayerTeam, kEscortTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
					}

					if ( Ship->mGetTeam() == kEscortTeam )
					{
						// if the Escort team takes heavy casualties, they will offer surrender
						if ( EscortTeam->mGetTotalDamagePercent() > kEscortTeamSurrenderThreshold )
						{
							// LNF I could make them surrender directly, or make the player have to push a button
						}
					}
				}
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					// Set team relations
					fMission.mSetTeamRelations (kPlayerTeam, kPlagueTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kQShipTeam, kTeamHostile, kGoalDestroy, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kEscortTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlayerTeam, kPlanetTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kPlagueTeam, kQShipTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlagueTeam, kEscortTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kPlagueTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kQShipTeam, kEscortTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);
					fMission.mSetTeamRelations (kQShipTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mSetTeamRelations (kEscortTeam, kPlanetTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue);

					fMission.mDisplayMessage( kStartScenario_msg, kMissionScriptMessage );
					fMission.mSetTimedInterval( kTimerLimitSet_2, 5 );
					break;

				case kTimerLimitSet_2:
					fMission.mDisplayMessage( kLongRangeScanFlotilla_msg, kMissionScriptMessage );
					fMission.mSetTimedInterval( kTimerLimitSet_3, 7 ); // timer to turn on/off goal indicator
					break;

				case kTimerLimitSet_3:
					{
						tShipInfo* DistanceShip;
						tShipInfo* PlayerTeamShip;
						bool WithinVisualRange = false;
						// timer to turn on/off goal indicator
						if ( GoalIndicator == kOff )
						{
							fMission.mDisplayMessage( kGoalIndicatorOn_msg, kMissionScriptMessage );
							fMission.mSetGoalIndicator( kTrue, 3, 23 );
							GoalIndicator = kOn;
							fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
						}
						else
						{
							// check the enemy fleets to see if the player fleet is within range
							for (DistanceShip = PlagueTeam->mGetFirstShip(); DistanceShip != NULL; DistanceShip = PlagueTeam->mGetNextShip())
							{
								for (PlayerTeamShip = PlayerTeam->mGetFirstShip(); PlayerTeamShip != NULL; PlayerTeamShip = PlayerTeam->mGetNextShip())
								{
									if ( PlayerTeamShip->mGetTacticalDistance(DistanceShip) < kDistanceToPlagueShips )
									{
										WithinVisualRange = true;
										break;
									}
								}
							}
							for (DistanceShip = QShipTeam->mGetFirstShip(); DistanceShip != NULL; DistanceShip = QShipTeam->mGetNextShip())
							{
								for (PlayerTeamShip = PlayerTeam->mGetFirstShip(); PlayerTeamShip != NULL; PlayerTeamShip = PlayerTeam->mGetNextShip())
								{
									if ( PlayerTeamShip->mGetTacticalDistance(DistanceShip) < kDistanceToPlagueShips )
									{
										WithinVisualRange = true;
										break;
									}
								}
							}
							for (DistanceShip = EscortTeam->mGetFirstShip(); DistanceShip != NULL; DistanceShip = EscortTeam->mGetNextShip())
							{
								for (PlayerTeamShip = PlayerTeam->mGetFirstShip(); PlayerTeamShip != NULL; PlayerTeamShip = PlayerTeam->mGetNextShip())
								{
									if ( PlayerTeamShip->mGetTacticalDistance(DistanceShip) < kDistanceToPlagueShips )
									{
										WithinVisualRange = true;
										break;
									}
								}
							}

							if ( WithinVisualRange == true )
							{
								if ( PlayerTeam->mGetShipsInPlayCount() > 1 )
								{
									fMission.mDisplayMessage( kGoalIndicatorOffMultiple_msg, kMissionScriptMessage );
								}
								else
								{
									fMission.mDisplayMessage( kGoalIndicatorOffSingle_msg, kMissionScriptMessage );
								}
								fMission.mSetGoalIndicator( kFalse, 0, 0 );
							}
							else
							{
								fMission.mSetTimedInterval( kTimerLimitSet_3, 1 );
							}							
						}
					}
					break;

				case kTimerTurnsSet_10: // COMM Button timer
					// set Plague the ship's comm buttons back to HAIL
					for (Ship = PlagueTeam->mGetFirstShip(); Ship != NULL; Ship = PlagueTeam->mGetNextShip())
					{
						Ship->mClearCommButton (2);
						Ship->mSetCommButton (1, kButtonHail_text);
					}
					break;
			}
			break;

		case kNotifyOnCapture:
			{
				tShipInfo* CapturingShip = fMission.mFindShipID( fEventID );
				
				fMission.mAddToBonusPrestige( kCaptureBonus );

				CheckTeamDestruction( *this, fMission.mGetTeamHandle( Ship->mGetTeam() ), kEventShipCaptured, Ship, CapturingShip );			

				return CheckWinCondition( *this );
/*
				if (Ship->mGetTeam() == kPlagueTeam)
				{
					PlagueDestroyed++;
					if (PlagueTeam->mGetShipsInPlayCount() == 0)
					{
					}
				}
				else if (Ship->mGetTeam() == kQShipTeam)
				{
					QShipDestroyed++;
					if (QShipTeam->mGetShipsInPlayCount() == 0)
					{
						AllQShipsDestroyed = 1;
					}
				}
				else if (Ship->mGetTeam() == kEscortTeam)
				{
					EscortDestroyed++;
					if (EscortTeam->mGetShipsInPlayCount() == 0)
					{
					}
				}
*/
			}
			break;

		case kNotifyAtDeath:

			CheckTeamDestruction( *this, fMission.mGetTeamHandle( Ship->mGetTeam() ), kEventShipDestroyed, Ship );

			return CheckWinCondition( *this );
/*
			if (Ship->mGetTeam() == kPlayerTeam)
			{
				PlayerDestroyed++;
				if (PlayerTeam->mGetShipsInPlayCount() == 0)
				{
					return kPlayerDestroyed;
				}
			}
			else if (Ship->mGetTeam() == kPlagueTeam)
			{
				PlagueDestroyed++;
				if (PlagueTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
			else if (Ship->mGetTeam() == kQShipTeam)
			{
				QShipDestroyed++;
				if (QShipTeam->mGetShipsInPlayCount() == 0)
				{
					AllQShipsDestroyed = 1;
				}
			}
			else if (Ship->mGetTeam() == kEscortTeam)
			{
				EscortDestroyed++;
				if (EscortTeam->mGetShipsInPlayCount() == 0)
				{
				}
			}
*/
			break;

		case kNotifyDisengaged:
			// LNF determine which ships escape
			if (Ship == fPlayerShip)
			{
				EscapedShips += ((PlagueTeam->mGetShipsInPlayCount()) +
								QShipTeam->mGetShipsInPlayCount() +
								EscortTeam->mGetShipsInPlayCount());
			}
			else if (Ship->mGetTeam() != kPlayerTeam)
			{
				EscapedShips++;
			}
			break;

		case kNotifyPlayerLeaving:
			PlayerUsedESCKey = true;
			break;

		case kNotifyCommButtonPress:
			{
				switch (fEventID)
				{
					case kButtonHail_text:
						switch (Ship->mGetTeam())
						{
							case kPlagueTeam:
								if (AtWarWithPlague == 0)
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kPlagueHailNic1_msg, kPlagueHailNic3_msg), kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kPlagueHailCombat1_msg, kPlagueHailCombat3_msg), kMissionScriptMessage);
									Ship->mSetCommButton (1, kButtonTaunt_text);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kQShipTeam:
								if (AtWarWithPlague == 0)
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kPlagueHailNic1_msg, kPlagueHailNic3_msg), kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kQShipHail1_msg, kQShipHail3_msg), kMissionScriptMessage);
									Ship->mSetCommButton (1, kButtonTaunt_text);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kEscortTeam:
								if (AtWarWithPlague == 0)
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kEscortHail1_msg, kEscortHail3_msg), kMissionScriptMessage);
								}
								else
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kPlagueHailCombat1_msg, kPlagueHailCombat3_msg), kMissionScriptMessage);
									Ship->mSetCommButton (1, kButtonTaunt_text);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								// If the Taunt or Surrender buttons aren't used in X turns
								// return the buttons to Hail.
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;
						}
						break;

					case kButtonSurrender_text:
						switch (Ship->mGetTeam())
						{
							case kPlagueTeam:

								// LNF I should also make the win condition a percentage of the Escort ships.
								// LNF mGetTotalDamagePercent()

								if (AllQShipsDestroyed == 0)
								{
									fMission.mDisplayMessage(kSurrender_msg, kMissionScriptMessage);
								}
								else if ((AllQShipsDestroyed == 1) && (Elite == kPraiex))
								{
									fMission.mDisplayMessage(kPraiexSurrender1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kPraiexSurrender2_msg, kMissionScriptMessage);
									fMission.mSetTeamRelations (kPlayerTeam, kPlagueTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
									fMission.mSetTeamRelations (kPlayerTeam, kEscortTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
									Surrender = 1;
									PlagueShipsStopped = true;
									return kPlayerWins;
									// LNF maybe I should turn off the ai on the ships to stop them?
									//fMission.mSetTimedInterval (kTimerLimitSet_2, 5);
								}
								else if ((AllQShipsDestroyed == 1) && (Elite == kShiar))
								{
									fMission.mDisplayMessage(kShiarSurrender1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kShiarSurrender2_msg, kMissionScriptMessage);
									fMission.mSetTeamRelations (kPlayerTeam, kPlagueTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
									fMission.mSetTeamRelations (kPlayerTeam, kEscortTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
									Agreement = 1;
									OrbitPlanet(PlagueTeam);
									OrbitPlanet(EscortTeam);
									PlagueShipsStopped = true;
									return kPlayerWins;
									//fMission.mSetTimedInterval (kTimerLimitSet_2, 5);
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kQShipTeam:
								fMission.mDisplayMessage(kSurrender_msg, kMissionScriptMessage);
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kEscortTeam:
								if (AllQShipsDestroyed == 0)
								{
									fMission.mDisplayMessage(kSurrender_msg, kMissionScriptMessage);
								}
								else if ((AllQShipsDestroyed == 1) && (Elite == kPraiex))
								{
									fMission.mDisplayMessage(kPraiexSurrender1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kPraiexSurrender2_msg, kMissionScriptMessage);
									fMission.mSetTeamRelations (kPlayerTeam, kPlagueTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
									fMission.mSetTeamRelations (kPlayerTeam, kEscortTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
									Surrender = 1;
									PlagueShipsStopped = true;
									return kPlayerWins;
									//fMission.mSetTimedInterval (kTimerLimitSet_2, 5);
								}
								else if ((AllQShipsDestroyed == 1) && (Elite == kShiar))
								{
									fMission.mDisplayMessage(kShiarSurrender1_msg, kMissionScriptMessage);
									fMission.mDisplayMessage(kShiarSurrender2_msg, kMissionScriptMessage);
									fMission.mSetTeamRelations (kPlayerTeam, kPlagueTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
									fMission.mSetTeamRelations (kPlayerTeam, kEscortTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue);
									Agreement = 1;
									OrbitPlanet(PlagueTeam);
									OrbitPlanet(EscortTeam);
									PlagueShipsStopped = true;
									return kPlayerWins;
									//fMission.mSetTimedInterval (kTimerLimitSet_2, 5);
								}
								Ship->mSetCommButton (2, kButtonSurrender_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;
						}
						break;

					case kButtonTaunt_text:
						switch (Ship->mGetTeam())
						{
							case kPlagueTeam:
								if (AtWarWithPlague == 1)
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kTaunt1_msg, kTaunt5_msg), kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kQShipTeam:
								if (AtWarWithPlague == 1)
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kTaunt1_msg, kTaunt5_msg), kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;

							case kEscortTeam:
								if (AtWarWithPlague == 1)
								{
									fMission.mDisplayMessage (fMission.mRandomInt32 (kTaunt1_msg, kTaunt5_msg), kMissionScriptMessage);
								}
								Ship->mSetCommButton (1, kButtonTaunt_text);
								fMission.mSetTimedInterval (kTimerTurnsSet_10, 2);
								break;
						}
						break;
				}
			}
			break;

	}

	// test to expell player from the elite clan
	eGameStatus status = fMission.mGetGameStatus();
	if (status != kGameInProgress)
	{
		// LNF change this victory thing for draw if you want.
		if (mCalcVictoryStatus() >= kDefeat)
		{
			status = eGameStatus(status | kExpellFromElite);
		}
	}
	return status;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	eVictoryLevels MyVictoryLevel = kDraw;

	int32 VictoryPoints = 0;
	const int32 BasePoints = 100;

	VictoryPoints = BasePoints - (PlagueDestroyed * 10) - (EscortDestroyed * 10) - (EscapedShips * 15) -
					(PlagueShipEscaped * 100);

	if (Elite == kPraiex)
	{
		VictoryPoints += (QShipDestroyed * 10);
	}
	else
	{
		VictoryPoints += (Agreement * 30);
	}

	if (VictoryPoints >= 100)
	{
		MyVictoryLevel = kAstoundingVictory;
	}
	else if ((VictoryPoints <= 99) && (VictoryPoints >= 70))
	{
		MyVictoryLevel = kVictory;
	}
	else if ((VictoryPoints <= 69) && (VictoryPoints >= 50))
	{
		MyVictoryLevel = kDraw;
	}
	else if (VictoryPoints <= 49)
	{
		MyVictoryLevel = kDevastatingDefeat;
	}

	// LNF: Add Dynaverse rating adjustments

	return (MyVictoryLevel);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	int32 SuccessCount = 0;
	int32 FailureCount = 0;

	if ( PlayerUsedESCKey == false )
	{
		if (fMission.mGetEliteOrganization() == kPraiex)
		{
			if (VictoryLevel == kAstoundingVictory)
			{
				fMission.mAddToDebriefingList(kPraiexAstoundingVictory_msg);
				fMission.mAddToDebriefingList(kVictory2_msg);
			}
			else if (VictoryLevel == kVictory)
			{
				fMission.mAddToDebriefingList(kPraiexVictory_msg);
				fMission.mAddToDebriefingList(kVictory2_msg);
			}
			else
			{
				fMission.mAddToDebriefingList (VictoryLevel + kPraiexAstoundingVictory_msg);
			}
		}
		else // Elite == kShiar
		{
			if (VictoryLevel == kAstoundingVictory)
			{
				fMission.mAddToDebriefingList(kShiarAstoundingVictory_msg);
				fMission.mAddToDebriefingList(kVictory2_msg);
			}
			else if (VictoryLevel == kVictory)
			{
				// PraiexVictory_msg used for both factions
				fMission.mAddToDebriefingList(kPraiexVictory_msg); 
				fMission.mAddToDebriefingList(kVictory2_msg);
			}
			else
			{
				fMission.mAddToDebriefingList (VictoryLevel + kShiarAstoundingVictory_msg);
			}
		}

		//************  this handles the Expulsion
		if ( VictoryLevel >= kDefeat )
		{
			if ( Elite == kPraiex )
			{
				fMission.mAddToDebriefingList( kPraiexExpelled_msg );
			}
			else // Elite == kShair
			{
				fMission.mAddToDebriefingList( kShiarExpelled_msg );
			}
			// LNF try this out:  adding results of primary objectives
			fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

			if ( Elite == kPraiex )
			{
				( PlagueShipsStopped == true ) ? fMission.mAddToDebriefingList( kPraiexPlagueShipsStoppedTrue_msg ) : SuccessCount++;
				if ( SuccessCount == 1 )
				{
					fMission.mAddToDebriefingList( kNone_msg );
				}

				fMission.mAddToDebriefingList( kFailedObjectives_msg );

				( PlagueShipsStopped == false ) ? fMission.mAddToDebriefingList( kPraiexPlagueShipsStoppedFalse_msg ) : FailureCount++;
				if ( FailureCount == 1 )
				{
					fMission.mAddToDebriefingList( kNone_msg );
				}
			}
			else // Shiar win conditions
			{
				( PlagueShipsStopped == true ) ? fMission.mAddToDebriefingList( (AllEnemyShipsDestroyed ? kShiarEnemyShipsDestroyedTrue_msg : kShiarPlagueShipsStoppedTrue_msg) ) : SuccessCount++;
				if ( SuccessCount == 1 )
				{
					fMission.mAddToDebriefingList( kNone_msg );
				}

				fMission.mAddToDebriefingList( kFailedObjectives_msg );

				( PlagueShipsStopped == false ) ? fMission.mAddToDebriefingList( kShiarPlagueShipsStoppedFalse_msg ) : FailureCount++;
				if ( FailureCount == 1 )
				{
					fMission.mAddToDebriefingList( kNone_msg );
				}
			}

			return;
		}
		//****************** this handles the expulsion

		fMission.mAddToDebriefingList (kDismissedShort_msg);
	}
	else // player exited improperly, display a message
	{
		fMission.mAddToDebriefingList( kImproperlyExitedMission_msg );
	}


	// LNF try this out:  adding results of primary objectives
	fMission.mAddToDebriefingList( kSuccessfulObjectives_msg );

	if ( Elite == kPraiex )
	{
		( PlagueShipsStopped == true ) ? fMission.mAddToDebriefingList( kPraiexPlagueShipsStoppedTrue_msg ) : SuccessCount++;
		if ( SuccessCount == 1 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}

		fMission.mAddToDebriefingList( kFailedObjectives_msg );

		( PlagueShipsStopped == false ) ? fMission.mAddToDebriefingList( kPraiexPlagueShipsStoppedFalse_msg ) : FailureCount++;
		if ( FailureCount == 1 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}
	else // Shiar win conditions
	{
		( PlagueShipsStopped == true ) ? fMission.mAddToDebriefingList( (AllEnemyShipsDestroyed ? kShiarEnemyShipsDestroyedTrue_msg : kShiarPlagueShipsStoppedTrue_msg) ) : SuccessCount++;
		if ( SuccessCount == 1 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}

		fMission.mAddToDebriefingList( kFailedObjectives_msg );

		( PlagueShipsStopped == false ) ? fMission.mAddToDebriefingList( kShiarPlagueShipsStoppedFalse_msg ) : FailureCount++;
		if ( FailureCount == 1 )
		{
			fMission.mAddToDebriefingList( kNone_msg );
		}
	}


	fMission.mScheduleMission( kNextMission_msg );
	// LNF modify the number of spacer missions?
	int32 NumberOfSpacerMissions = fMission.mRandomInt32( 0, 2 );
	switch (NumberOfSpacerMissions)
	{
		case 0:
			break;

		case 1:
			fMission.mScheduleMission( kRandomMission_msg );
			break;
			
		case 2:
			fMission.mScheduleMission( kRandomMission_msg );
			fMission.mScheduleMission( kRandomMission_msg );
			break;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every end of tutorial or technical mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& infovoid)
{
	return 0;
}

//---------------------------------------------------------------------------------------------
// make the ships orbit the planet
void OrbitPlanet(tTeamInfo* OrbitTeam)
{
	tShipInfo* Ship;

	tMapPoint waypoints[6]=
	{
		28, 13,
		32, 13,
		34, 10,
		32, 7,
		28, 7,
		26, 10
	};

	tMapPoint PatrolPoints[6];

	for (Ship = OrbitTeam->mGetFirstShip(); Ship != NULL; Ship = OrbitTeam->mGetNextShip())
	{
		PatrolPoints[0] = waypoints[0];
		PatrolPoints[1] = waypoints[1];
		PatrolPoints[2] = waypoints[2];
		PatrolPoints[3] = waypoints[3];
		PatrolPoints[4] = waypoints[4];
		PatrolPoints[5] = waypoints[5];
		Ship->mPatrol (1.0, 6, &PatrolPoints[0], kHighPriority);
	}
}

//---------------------------------------------------------------------------------------------
// win condition
eGameStatus CheckWinCondition( tScript& tScript )
{
	if (( AllPlagueShipsDestroyed == 1 ) &&
		( AllQShipsDestroyed == 1 ) &&
		( AllEscortsDestroyed == 1 ))
	{
		PlagueShipsStopped = true;
		AllEnemyShipsDestroyed = true;
		return eGameStatus( kPlayerWins );
	}
	else
	{
		return kGameInProgress;
	}
/*
			tScript.fMission.mDisplayMessage( kPrimaryObjectivesComplete_msg, kMissionScriptMessage );
			FindClosestMapEdge( tScript, tScript.fPlayerShip, kMapEdgeX, kMapEdgeY );
*/
}

//---------------------------------------------------------------------------------------------
// check if a team has all ships destroyed/captured/disengaged
void CheckTeamDestruction(tScript& tScript, tTeamInfo* EventTeam, eEventShipStatus EventShipStatus,
						  tShipInfo* Ship, tShipInfo* CapturingShip)
{
	tTeamInfo* PlayerTeam = tScript.fMission.mGetTeamHandle (tScript.kPlayerTeam);
	tTeamInfo* PlagueTeam = tScript.fMission.mGetTeamHandle (kPlagueTeam);
	tTeamInfo* QShipTeam = tScript.fMission.mGetTeamHandle (kQShipTeam);
	tTeamInfo* EscortTeam = tScript.fMission.mGetTeamHandle (kEscortTeam);
	tTeamInfo* PlanetTeam = tScript.fMission.mGetTeamHandle (kPlanetTeam);

	// if all ships on a team are gone
	//if ( EventTeam->mGetShipsInPlayCount() == ((EventShipStatus == kEventShipCaptured) ? 1 : 0) )
	if (EventTeam == PlagueTeam)
	{
		PlagueDestroyed++;
		if (EventTeam->mGetShipsInPlayCount() == 0)
		{
			AllPlagueShipsDestroyed = 1;
		}
	}
	else if (EventTeam == QShipTeam)
	{
		QShipDestroyed++;
		if (EventTeam->mGetShipsInPlayCount() == 0)
		{
			AllQShipsDestroyed = 1;
		}
	}
	else if (EventTeam == EscortTeam)
	{
		EscortDestroyed++;
		if (EventTeam->mGetShipsInPlayCount() == 0)
		{
			AllEscortsDestroyed = 1;
		}
	}
}
