#include "scriptinterface.h"				// must have this as the first include

#include "MissionText.h"					// must have these two includes
#include "MissionMaps.h"

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-human player teams in the script
// (use kPlayerTeam to access the current human players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for human player teams.
	kReserved = kTeam6,

	// add all non-human player teams (enemy, allied, monsters, etc...) after this point.

	kAlliedTeam,
	kEnemyTeam,
	kEnemyTeamThatAppearsLater,
};

// -----------------------------------------------------------------------------------
// this area can be used to add extra enums, work buffers or whatever is needed

tMapPoint waypoints[5]=
{
	5, 10,
	7, 23,
	14, 23,
	3, 21,
	4, 6
};

tMapPoint PatrolPoints[5][10];

// -----------------------------------------------------------------------------------

// include this file after all other includes
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

// -----------------------------------------------------------------------------------

//---------------------------------------------------------------------------------------------
//			START OF SCRIPT ROUTINES
//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaign list
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kHistoricalMission;				// use values from eMissionTypes enum in missionspec.h
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;					// use values from eMissSpecSect enum in missionspec.h
	Specs->fPlayerRace = kAllRaceMission;				// use values from eMissSpecRace enum in missionspec.h
	Specs->fEnemyRace = kAllRaceMission;				// use values from eMissSpecRace enum in missionspec.h
	Specs->fNumShips = MissSpecAllShips;				// use values from Bit masks in missionspec.h
	Specs->fMinBPV = 0;									// player must have at least this much BPV to play this mission
	Specs->fMaxBPV = kMaxBPV;							// player cannot have more than this much BPV to play this mission
	Specs->fMinDate = 9999;								// player can play this mission starting on this year date (date starts at 0
														//   and advances 1 year at a time)
	Specs->fMaxDate = 9999;								// player will not be able to play this mission after this year date
	Specs->fWait = kUniqueMission;						// has multiple uses depending on which fMissionType was specified above:
														// for kCampaignMission:
														//		specifies how many years Dynaverse will wait before selecting 
														//		this misison again. when fWait == kUniqueMission then this mission
														//		is played only once per campaign.
														// for kTutorialMission & kTechnicalMission:
														//		specifies this missions position in the academy menu. must be 0 - 5.
														// for kHistoricalMission & kMultiplayerMission:
														//		unused - should be 0
	Specs->fEnableSupplyDock = kFalse;					// enables/disables the Supply dock button in the briefing screen.
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (60, 40, 20, 0, -5);

	// can change the race of enemy or ally if needed (but NOT player)
	if (info.fPlayerRace == kFederation
		&& info.fEnemyRace != kKlingon)
	{
		info.fEnemyRace = kKlingon;
	}

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, info.fEnemyRace);

	// must setup team relations and target goals
	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0);
	fMission.mSetTeamRelations (kEnemyTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission.
// can also change the default starting position of the ship (Ship.fShipPosition).
// all players ships start at kStartPosition_G and increament from there.
// (you are not able to specify an X/Y grid location)
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	Ship.fShipPosition += kStartPosition_P - kStartPosition_G;
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before the mission starts with Mode == 0.
// also called after the mission starts (Mode can be any user supplied value (expect 0)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);

	if (Mode == 0)
	{
		// create enemy star base
		EnemyTeam->mCreateShip ("K-SB", kStartPosition_S);
	}


	// create enemy ship(s)
	const char* name;
	if (EnemyTeam->mGetRace() == kFederation)
	{
		name = "F-CAR";
	}
	else
	{
		name = "K-D7K";
	}
	EnemyTeam->mCreateShip (name, kStartPosition_X);
//	EnemyTeam->mCreateShip (name, kStartPosition_Specified, 15, 25);
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return index to briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
//	char msg[200];

	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tShipInfo* Ship;
	int32 x, y;

	// configure player ship(s)
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for (Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip())
	{
		Ship->mSetShipThrust (10);
		Ship->mTrackTacticalDistance (50.0, 10, 30);			// setup notice at half-way point
//		Ship->mDamageSystem (kDamageWeaponsAndSystems, 10.0);
//		Ship->mDamageSystem (kDamageMaxShields, 50.0);
		Ship->mDamageSystem( kDamageScannerSystem, 100.0 );
		Ship->mDamageSystem( kDamageTractorSystem, 100.0 );
		Ship->mDamageSystem( kDamageShuttleBaySystem, 100.0 );
		Ship->mDamageSystem( kDamageTransporterSystem, 100.0 );
		Ship->mSetShipCallbackEvents (kNotifyTransportedItemTo);
		Ship->mAccessSupplies (kSetSupplyCount, kSupplyShuttles, 2);
		x = Ship->mAccessSupplies (kGetHeavyWeaponCount, kDamageHeavyWeapon10);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransMarine, 1);
//		Ship->mAccessSupplies (kSetTransporterItemCount, kTransGravTank, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransCaptain, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransGornEgg, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransCrew, 1);
//		Ship->mAccessSupplies (kSetTransporterItemCount, kTransHeavyWeaponPlatform, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransSpy, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransSparePart, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransScientists, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransMedicalSupplies, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransTribbles, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransRoyalGornEgg, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransDiplomat, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransCommandos, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransRomulanAle, 1);
//		Ship->mAccessSupplies (kSetTransporterItemCount, kTransGenericPiratedCargo, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransDilithiumCrystals, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransBlackBox, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransDeathPlague, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransHarryMudd, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransInfiltrator, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransIonicProjector, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransLifePodDude, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransMedicineJar, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransNovaMine, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransPsionicDisruptor, 1);
		Ship->mAccessSupplies (kSetTransporterItemCount, kTransWeaponSchematic, 1);

		Ship->mSetTargetable (kFalse);
		Ship->mSetTargetable (kTrue);
	}

	for (Ship = EnemyTeam->mGetFirstShip (); Ship != NULL; Ship = EnemyTeam->mGetNextShip ())
	{
		Ship->mSetShipTarget (PlayerTeam->mGetFirstShip (), kVeryHighPriority);
		Ship->mGetGridPosition (&x, &y);
		float dist = Ship->mGetTacticalDistance (PlayerTeam->mGetFirstShip());
		dist = Ship->mGetTacticalDistance ('1');
		dist = Ship->mGetTacticalDistance (15, 30);

		Ship->mSetCommButton (1, kButton1_Text);
		Ship->mSetCommButton (2, kButton2_Text);
	}

//	fMission.mSetTimedInterval (kTimerLimitSet, 20);	// set to be notified in 20 seconds
//	fMission.mSetTimedInterval (kTimerTurnsSet, 10);	// set to be notified in 10 turns
//	fMission.mSetTimedInterval (kTimerCountdownSet, 5);	// set countdown timer to expire in 5 turns

	fMission.mGetGridPosition ('7', &x, &y);
	float dist = fMission.mGetTacticalDistance ('5', '3');

//	fMission.mCreateMapObject ('A', 10, 33);

	int32 rank = fMission.mGetPlayerRank();
	int32 elite = fMission.mGetEliteOrganization();

	fMission.mPlaySound (kSndMineDrop);

//	Ship = EnemyTeam->mGetFirstShip ();
//	sprintf (msg, "enemy class: %i, BPV: %i", Ship->mGetClassType(), Ship->mGetShipBPV());
//	fMission.mDebugMessage (msg);

	fMission.mStartFlashingButton ("A3_OFFICER_REPAIR");

	PatrolPoints[0][0] = waypoints[0];
	PatrolPoints[0][1] = waypoints[3];
	PatrolPoints[1][0] = waypoints[2];
	PatrolPoints[1][1] = waypoints[4];
	PatrolPoints[2][0].x = 10;
	PatrolPoints[2][0].y = 15;

//	Ship->mPatrol (10.0, 2, &PatrolPoints[1][0], kAbsolutePriority);

	// setup to test deep scan every second - must be called before any ships are added
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer
	// deep scan enemy star base from within distance 20 for 4 turns
	DeepScanTracker.mSetTarget (EnemyTeam->mGetThisShip(1), 20.0, 4);
	DeepScanTracker.mSetTarget (EnemyTeam->mGetThisShip(2), 10.0, 4);

	fMission.mSetTimedInterval( kTimerLimitSet_7, 4 );
	fMission.mSetTimedInterval( kTimerLimitSet_8, 6 );
	fMission.mSetTimedInterval( kTimerLimitSet_9, 10 );
}

//---------------------------------------------------------------------------------------------
// should return whatever team is considered to be the primary enemy.
// NOTE: should only return values found in eTeamTypes enum above
int32 tScript::mGetEnemyTeamID (void)
{
	return kEnemyTeam;
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	if (Ship->mGetTeam() == kPlayerTeam)
	{
		Ship->mSetShipPlayerControlled();
	}
	else
	{
		tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
		Ship->mSetShipTarget (PlayerTeam->mGetFirstShip (), kVeryHighPriority);
	}
}

//---------------------------------------------------------------------------------------------
// called during the mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);

	tShipInfo* target = fMission.mGetShipTarget(fPlayerShip);

	switch (Event)
	{
		case kNotifyTransportedItemTo:
			if (fEventItem1 == kTransSparePart)
			{
			}
			break;

		case kNotifyFlashingButtonPressed:
			fMission.mStartFlashingButton ("A3_OFFICER_COMM");
			fPlayerShip->mDamageSystem (kDamageInternalsWithPoints, 30.0);
			break;

		case kNotifyAtDeath:
			if (Ship->mGetTeam() == kEnemyTeam)
			{
				fMission.mAddToCareerTotals (Ship->mGetClassType(), kDestroyed, 1);
			}
			break;

		case kNotifyAtDistance:
			if (Ship->mGetTrackingDistance() == 50.0)			// test for half-way point
			{
				fMission.mDisplayMessage (kmsg1);			// display half-way message
				Ship->mTrackTacticalDistance (10.0, 10, 30);			// reset notice to just in front of sun
				eAlertLevels level = Ship->mGetAlertStatus();
			}
			else	// we must be at our final location
			{
//				Ship->mSetShipThrust (0);					// shut down thrust
//				fMission.mSetTimedInterval (kTimerLimitSet, 0);	// turn off timer callback
//				mCreateMissionShips (1);					// suprise player with enemy ships
			}
			break;

		case kNotifyAtTimedInterval:
			switch (fEventID)
			{
				case kTimerLimitSet:
					fMission.mDisplayMessage (kmsg2);				// let the player know he's taking too long		
//					fMission.mSetTimedInterval (kTimerLimitSet, 20);// reset timer to be notified every 20 seconds
					break;

				case kTimerTurnsSet:
//					fMission.mSetTimedInterval (kTimerTurnsSet, 2);
					break;

				case kTimerLimitSet_7:
					fMission.mDisplayMessage( kMessageNonComm0_msg + fMission.mRandomInt32( 0, 3 ), kNonCommScriptMessage );
					fMission.mSetTimedInterval( kTimerLimitSet_7, fMission.mRandomInt32( 5, 15 ) );
					break;

				case kTimerLimitSet_8:
					switch( fMission.mRandomInt32( 1, 4 ) )
					{
						case 1:
							fMission.mDisplayMessage( kMessageEngineer0_msg + fMission.mRandomInt32( 0, 3 ), kEngineerScriptMessage );
							break;

						case 2:
							fMission.mDisplayMessage( kMessageMission0_msg + fMission.mRandomInt32( 0, 3 ), kMissionScriptMessage );
							break;

						case 3:
							fMission.mDisplayMessage( kMessageScience0_msg + fMission.mRandomInt32( 0, 3 ), kScienceScriptMessage );
							break;

						case 4:
							fMission.mDisplayMessage( kMessageFleet0_msg + fMission.mRandomInt32( 0, 3 ), kFleetScriptMessage );
							break;
					}

					fMission.mSetTimedInterval( kTimerLimitSet_8, fMission.mRandomInt32( 5, 15 ) );
					break;

				case kTimerLimitSet_9:
					fMission.mDisplayMessage( kNuggies_msg );
					fMission.mSetTimedInterval( kTimerLimitSet_9, fMission.mRandomInt32( 30, 60 ) );
					break;

				case kTimerLimitSet_10:
					{
						eDeepScanFlags flag;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag);
						switch (flag)
						{
							case kOutOfRange_ScanOff:
								// player is not within any objects range and deep scan is off (Object == NULL)
								break;
							case kOutOfRange_ScanOn:
								// player is not within any objects range and deep scan is on (Object == NULL)
								break;
							case kEnteringRange_ScanOff:
								// player has just entered range and scan is off (Object points to entity)
								// notify player to turn scan on
								fMission.mDisplayMessage (kDeepScan_TurnItOn);
								break;
							case kEnteringRange_ScanOn_TimerStarted:
								// player has just entered range and scan is on (Object points to entity)
								// notify player that timer has begun
								fMission.mDisplayMessage (kDeepScan_TimerBegun);
								break;
							case kInRange_ScanOff_TimerStopped:
								// player is in range and scan is off (Object points to entity)
								// notify player that timer has been turned off
								fMission.mDisplayMessage (kDeepScan_TimerTurnedOff);
								break;
							case kInRange_ScanOff:
								// player is still in range and scan is still off (Object points to entity)
								break;
							case kInRange_ScanOn_TimerStarted:
								// player is in range and scan is on (Object points to entity)
								// notify player that timer has begun
								fMission.mDisplayMessage (kDeepScan_TimerBegun);
								break;
							case kInRange_ScanOn:
								// player is still in range and scan is still on (Object points to entity)
								break;
							case kLeavingRange_ScanOn_TimerStopped:
								// player has just left the range and scan is on (Object points to entity)
								// notify player that timer is now off
								fMission.mDisplayMessage (kDeepScan_TimerTurnedOff);
								break;
							case kLeavingRange_ScanOff:
								// player has just left the range and scan is off (Object points to entity)
								break;
						}
					}
					break;

				case kTimerCountdownSet:
					{
						eDeepScanFlags flag;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag);
						if (flag == kInRange_ScanOn)
						{
							// player has successfully scanned the object
							fMission.mDisplayMessage (kDeepScan_ObjectScanned);
							DeepScanTracker.mRemoveTarget (Object);
						}
					}
					break;
			}
			break;

		case kNotifyCommButtonPress:
			switch (fEventID)
			{
				case kButton1_Text:
					// the third value of kTrue will force the players current ship to select this ship as
					// a target and immediately display the Comm panel (if in view)
					Ship->mSetCommButton (1, kButton1_Text2, kTrue);
					break;

				case kButton2_Text:
					Ship->mSetCommButton (2, kButton2_Text2);
					break;

				case kButton2_Text2:
					Ship->mSetCommButton (1, kButton1_Text);
					Ship->mSetCommButton (2, kButton2_Text);
					break;
			}
			break;

		case kNotifyDamage:
			{
				// get a handle to the ship that did the damage
				tShipInfo* HostileShip = fMission.mFindShipID (fEventID);
				if (HostileShip != NULL)
				{
					// do what's needed to the HostileShip or its team
				}
				else
				{
					// no ship matched - must have been debris that caused the damage
				}

				switch (fEventItem1)
				{
					case kDamageWeaponByMarines:
						// fEventItem2 == which weapon was hit: kDamageHeavyWeapon1 - kDamageHeavyWeapon25
						// fEventItem3 == weapons current health; 0 == weapon destroyed
						break;


					case kDamageSystemByMarines:
						// fEventItem2 == which system was hit: kDamageSensorSystem, 
						//										kDamageTransporterSystem,
						//										kDamageTractorSystem
						//										kDamageShuttleBaySystem
						// fEventItem3 == systems current health; 0 == system destroyed
						break;

					case kDamageByWeaponOrDebris:
						// nothing extra to test
						break;
				}
			}
			break;
	}
	eGameStatus status = fMission.mGetGameStatus();

	// need to test team total damage BEFORE returning status that will end the game
	if (status != kGameInProgress)
	{
		float damage = PlayerTeam->mGetTotalDamagePercent();
		damage = EnemyTeam->mGetTotalDamagePercent();
	}

	return status;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);

	// determine who gets the combat BPV
	int32 PlayerBPV = PlayerTeam->mGetCombatBPV();
	int32 EnemyBPV = EnemyTeam->mGetCombatBPV();
	if (PlayerBPV > EnemyBPV)
	{
		EnemyBPV = PlayerBPV - EnemyBPV;
		PlayerBPV = 0;
	}
	else
	{
		PlayerBPV = EnemyBPV - PlayerBPV;
		EnemyBPV = 0;
	}

	// add in assesed damage from the opposing team
	PlayerBPV += EnemyTeam->mAssessDamages(PlayerTeam);
	EnemyBPV += PlayerTeam->mAssessDamages(EnemyTeam);

	return (fMission.mGetVictoryLevel (PlayerBPV, EnemyBPV));
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);
	fMission.mAddToDebriefingList (kmsg1);
	fMission.mAddToDebriefingList (kmsg2);
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission only
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

