#include "scriptinterface.h"

#include "Tut_Hydran2Text.h"
#include "Tut_Hydran2Maps.h"

int32 TutorialSegment = 0;	// a counter that advances when the player completes part of the tutorial
bool MissionComplete = false;
bool ResupplyRequested = false;
bool TutorialPassed = false;
int32 MaxResupplies = 1;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam, kAlliedTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;						// to stop the mission from ending prematurely
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kTutorialMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kHydranMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 1;
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kAlliedTeam, kHydran );
	fMission.mSetTeamRace( kEnemyTeam, kLyran );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
	fMission.mSetTeamRelations( kPlayerTeam, kAlliedTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations( kEnemyTeam, kAlliedTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// we will not be using the normal player's ship this mission
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tShipInfo* Ship;

	int32 x, y;

	// Create a ship for the player to use
	if( Mode == 0 )
	{
		PlayerTeam->mCreateShip( "H-RNT", kStartPosition_G );
	}

	if( Mode == 1 )
	{
		Ship = PlayerTeam->mGetFirstShip();
		Ship->mGetGridPosition( &x, &y );
		EnemyTeam->mCreateShip( "L-CA", kStartPosition_Specified, x+3, y );
		AlliedTeam->mCreateShip( "H-FSPh", kStartPosition_Specified, x-1, y-2 );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tShipInfo* Ship;

	// configure the player's Ranger
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mChangeName( kHeadmaster_name );
	Ship->mDamageSystem( kDamageHeavyWeapon7, 100.0 );
	Ship->mDamageSystem( kDamageHeavyWeapon8, 100.0 );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplySpareParts, 0 );
	Ship->mAccessSupplies( kSetFighterItem, kFighterBay1GroupType, 1 );	// patrol
	Ship->mAccessSupplies( kSetFighterItem, kFighterBay2GroupType, 2 );	// interceptor
	Ship->mAccessSupplies( kSetFighterItem, kFighterBay3GroupType, 3 );	// heavy
	Ship->mAccessSupplies( kSetFighterItem, kFighterBay4GroupType, 4 );	// assault
	Ship->mSetShipCallbackEvents( kNotifyAlertStatusChange );

	fMission.mSetTimedInterval( kTimerLimitSet, 3 );	// set timer to display starting message
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	if( Ship->mGetTeam() == kAlliedTeam )
	{
		Ship->mSetCommButton( 1, kButtonResupply_text );
		Ship->mSetShipMaxSpeed( 0 );
	}

	Ship->mChangeAIOfficerState( kECMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kECCMOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTractorOfficer, kFalse );
	Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );
	Ship->mChangeAIOfficerState( kHitAndRunOfficer, kFalse );
	Ship->mChangeAIOfficerState( kTBombOfficer, kFalse );
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	switch (Event)
	{
		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					switch( TutorialSegment )
					{
						case 0:
							fMission.mDisplayMessage( kHydran2Where_msg, kNonCommScriptMessage );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 1:
							fMission.mDisplayMessage( kHydran2Where2_msg, kNonCommScriptMessage );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 2:
							fMission.mDisplayMessage( kHydran2Where3_msg, kNonCommScriptMessage );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 3:
							fMission.mDisplayMessage( kHydran2Welcome_msg );
							fMission.mPlayVoiceSample( kSndHydran2Welcome );
							if( fPlayerShip->mGetAlertStatus() == kAlertRed )
							{
								fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							}
							break;

						case 4:
							fMission.mDisplayMessage( kHydran2GeneralFighter_msg );
							fMission.mPlayVoiceSample( kSndHydran2GeneralFighter );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 5:
							fMission.mDisplayMessage( kHydran2Rumors_msg );
							fMission.mPlayVoiceSample( kSndHydran2Rumors );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 6:
							fMission.mDisplayMessage( kHydran2Squadrons_msg );
							fMission.mPlayVoiceSample( kSndHydran2Squadrons );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 7:
							fMission.mStartFlashingButton( "PlayerSchem.Shuttle" );
//							fMission.mStartFlashingButton( "INT_SHUTTLE_FIGHTER" );
							fMission.mDisplayMessage( kHydran2ClickShuttleBay_msg );
							fMission.mPlayVoiceSample( kSndHydran2ClickShuttleBay );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 8:
							fMission.mDisplayMessage( kHydran2FighterIcons_msg );
							fMission.mPlayVoiceSample( kSndHydran2FighterIcons );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 9:
							fMission.mStartFlashingButton( "Shuttle1" );
							fMission.mDisplayMessage( kHydran2ClickSquadron_msg );
							fMission.mPlayVoiceSample( kSndHydran2ClickSquadron );
							break;

						case 10:
							fMission.mStartFlashingButton( "A6_FIGHT_LAUNCH" );
							fMission.mDisplayMessage( kHydran2ClickLaunch_msg );
							fMission.mPlayVoiceSample( kSndHydran2ClickLaunch );
							break;

						case 11:
//							fMission.mDisplayMessage( kHydran2Orders_msg );
//							fMission.mPlayVoiceSample( kSndHydran2Orders );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 12:
							fMission.mDisplayMessage( kHydran2GiveOrders_msg );
							fMission.mPlayVoiceSample( kSndHydran2GiveOrders );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 13:
							fMission.mDisplayMessage( kHydran2AttackOrders_msg );
							fMission.mPlayVoiceSample( kSndHydran2AttackOrders );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 14:
							fMission.mDisplayMessage( kHydran2DefendOrders_msg );
							fMission.mPlayVoiceSample( kSndHydran2DefendOrders );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 15:
							fMission.mDisplayMessage( kHydran2ReturnOrders_msg );
							fMission.mPlayVoiceSample( kSndHydran2ReturnOrders );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 16:
							fMission.mDisplayMessage( kHydran2DeckCrews_msg );
							fMission.mPlayVoiceSample( kSndHydran2DeckCrews );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 17:
							mCreateMissionShips( 1 );
							fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue);
							fMission.mSetTeamRelations( kPlayerTeam, kAlliedTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
							fMission.mSetTeamRelations( kEnemyTeam, kAlliedTeam, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
							fMission.mDisplayMessage( kHydran2TargetLyran_msg );
							fMission.mPlayVoiceSample( kSndHydran2TargetLyran );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 18:
							fMission.mDisplayMessage( kHydran2ResupplyOrders_msg );
							fMission.mPlayVoiceSample( kSndHydran2ResupplyOrders);
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 19:
							fMission.mDisplayMessage( kHydran2Stinger1s_msg );
							fMission.mPlayVoiceSample( kSndHydran2Stinger1s );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 20:
							fMission.mDisplayMessage( kHydran2Stinger2s_msg );
							fMission.mPlayVoiceSample( kSndHydran2Stinger2s );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 21:
							fMission.mDisplayMessage( kHydran2StingerHs_msg );
							fMission.mPlayVoiceSample( kSndHydran2StingerHs );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 22:
							fMission.mDisplayMessage( kHydran2Cautions_msg );
							fMission.mPlayVoiceSample( kSndHydran2Cautions );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 23:
							fMission.mDisplayMessage( kHydran2Limitations_msg );
							fMission.mPlayVoiceSample( kSndHydran2Limitations );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 24:
							if( MissionComplete == true )
							{
								fMission.mDisplayMessage( kHydran2Dismissed_msg );
								fMission.mPlayVoiceSample( kSndHydran2Dismissed );
								fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							}
							else
							{
								MissionComplete = true;
							}
							break;

						default:
							return kPlayerWins;
					}
					break;

			case kTimerLimitSet_2:	// Loop until voice is done playing...
				if( fMission.mVoiceSampleStillPlaying() != 0 )
				{
					fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
				}
				else
				{
					TutorialSegment++;
					fMission.mSetTimedInterval( kTimerLimitSet, 1 );
				}
				break;

			case kTimerTurnsSet:
				if( ResupplyRequested == true )
				{
					if( AlliedTeam->mGetShipsInPlayCount() == 1 )
					{
						Ship = AlliedTeam->mGetFirstShip();

						if( Ship->mGetTacticalDistance( fPlayerShip ) <= 3 )
						{
							fPlayerShip->mAccessSupplies( kResetAllItems, 0, 0 );
							MaxResupplies--;
							ResupplyRequested = false;

							if( MaxResupplies <= 0 )
							{
								fMission.mDisplayMessage( kHydran2Finished_msg );
								Ship->mDisengage( kByMapEdge, kAbsolutePriority );
							}
							else
							{
								fMission.mDisplayMessage( kHydran2NotDone_msg );
								Ship->mSetCommButton( 1, kButtonResupply_text );
							}
						}
						else
						{
							Ship->mSetCommButton( 1, kButtonResupply_text );
						}
					}
				}
				break;
			}
			break;

		case kNotifyFlashingButtonPressed:
			{
				TutorialSegment++;
				fMission.mSetTimedInterval( kTimerLimitSet, 1 );
			}
			break;

		case kNotifyAlertStatusChange:
			if(( Ship->mGetAlertStatus() == kAlertRed ) && ( TutorialSegment >= 3 ))
			{
				TutorialSegment = 4;
				fMission.mSetTimedInterval( kTimerLimitSet, 1 );
			}
			else
			{
				Ship->mSetShipCallbackEvents( kNotifyAlertStatusChange );
			}
			break;

		case kNotifyCommButtonPress:
			if( Ship->mGetTacticalDistance( fPlayerShip ) <= 3 )
			{
				fMission.mDisplayMessage( kHydran2Roger_msg );
				fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
				ResupplyRequested = true;
			}
			else
			{
				fMission.mDisplayMessage( kHydran2NoGood_msg );
				Ship->mSetCommButton( 1, kButtonResupply_text );
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				fMission.mDisplayMessage( kHydran2PlayerDead_msg, kNonCommScriptMessage );
				fMission.mPlayVoiceSample( kSndHydran2PlayerDead );
				return kPlayerLoses;
			}
			else
			{
				if( MissionComplete == false )
				{
					MissionComplete = true;
				}
				else
				{
					fMission.mDisplayMessage( kHydran2LyranDead_msg );
					fMission.mPlayVoiceSample( kSndHydran2LyranDead );
					fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
				}
			}
			break;

		case kNotifyOnCapture:
			if( Ship->mGetTeam() == kEnemyTeam )
			{
				if( MissionComplete == false )
				{
					fMission.mDisplayMessage( kHydran2LyranCaptured_msg );
					fMission.mPlayVoiceSample( kSndHydran2LyranCaptured );
					TutorialSegment = 99;	// set it to the mission dismissed segment
					fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
				}
				else
				{
					MissionComplete = true;
				}
			}
			break;

		case kNotifyDisengaged:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				return kPlayerWins;
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	return (kAstoundingVictory);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	if( PlayerTeam->mGetShipsInPlayCount() == 0 )
	{
		fMission.mAddToDebriefingList( kBadDebrief_msg );
	}
	else
	{
		TutorialPassed = true;
		fMission.mAddToDebriefingList (kDebrief_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if( TutorialPassed == true )
	{
		return 1;
	}

	return 0;
}
