#include "scriptinterface.h"

#include "Tut_Romulan2Text.h"
#include "Tut_Romulan2Maps.h"

int32 TutorialSegment = 0;	// a counter that advances when the player completes part of the tutorial
bool TutorialPassed = false;
bool MissionEnd = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;						// to stop the mission from ending prematurely
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kTutorialMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kRomulanMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 1;							// second choice in menu
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, kRomulan );
	fMission.mSetTeamRace( kEnemyTeam, kMonster );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// we will not be using the normal player's ship this mission
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );

	// Create a ship for the player to use
	PlayerTeam->mCreateShip( "R-WE", kStartPosition_G );

	// Create the enemy defensive satellites
	EnemyTeam->mCreateShip( "K-DEF", kStartPosition_H );
	EnemyTeam->mCreateShip( "K-DEF", kStartPosition_I );
	EnemyTeam->mCreateShip( "K-DEF", kStartPosition_J );
	EnemyTeam->mCreateShip( "K-DEF", kStartPosition_K );
	EnemyTeam->mCreateShip( "K-DEF", kStartPosition_L );
//	EnemyTeam->mCreateShip( "K-DEF", kStartPosition_M );
	EnemyTeam->mCreateShip( "K-DEF", kStartPosition_N );
	EnemyTeam->mCreateShip( "K-DEF", kStartPosition_O );
	EnemyTeam->mCreateShip( "K-DEF", kStartPosition_P );
	EnemyTeam->mCreateShip( "K-DEF", kStartPosition_Q );
//	EnemyTeam->mCreateShip( "K-DEF", kStartPosition_R );
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tShipInfo* Ship;

	// configure the player's ship
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mSetShipThrust( 0 );
	Ship->mChangeName( kChild_name );
	Ship->mAccessSupplies( kSetSupplyCount, kSupplySpareParts, 0 );
	Ship->mTrackTacticalDistance( 20.0, 10, 19 );		// track distance to planet
	Ship->mSetShipCallbackEvents( kNotifyAlertStatusChange );
	Ship->mDamageSystem( kDamageCloakSystem, 100.0 );

	for( Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip() )
	{
		Ship->mChangeName( kDefSat_name );
	}
	fMission.mSetTimedInterval( kTimerLimitSet, 1 );	// set timer to display starting message
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					switch( TutorialSegment )
					{
						case 0:
							fMission.mDisplayMessage( kWhere_msg, kNonCommScriptMessage );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 1:
							fMission.mDisplayMessage( kWhere2_msg, kNonCommScriptMessage );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 2:
							fMission.mDisplayMessage( kWhere3_msg, kNonCommScriptMessage );
							TutorialSegment++;
							fMission.mSetTimedInterval( kTimerLimitSet, 1 );
							break;

						case 3:
							fMission.mDisplayMessage( kRom2Welcome_msg );
							fMission.mPlayVoiceSample( kSndRom2Welcome );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 4:
							fMission.mDisplayMessage( kRom2Warning_msg );
							fMission.mPlayVoiceSample( kSndRom2Warning );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 5:
							fMission.mDisplayMessage( kRom2RedAlert_msg );
							fMission.mPlayVoiceSample( kSndRom2RedAlert );

							if( fPlayerShip->mGetAlertStatus() == kAlertRed )
							{
								fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							}
							break;

						case 6:
							fMission.mDisplayMessage( kRom2CloakLocation_msg );
							fMission.mPlayVoiceSample( kSndRom2CloakLocation );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 7:
							fMission.mStartFlashingButton( "A3_OFFICER_ENERGY" );
							fMission.mDisplayMessage( kRom2ClickEnergy_msg );
							fMission.mPlayVoiceSample( kSndRom2ClickEnergy );
							fPlayerShip->mDamageSystem( kDamageCloakSystem, 0.0 );
							break;

						case 8:
							fMission.mStartFlashingButton( "cCloakingDevice" );
							fMission.mDisplayMessage( kRom2ClickCloakingButton_msg );
							fMission.mPlayVoiceSample( kSndRom2ClickCloakingButton );
							break;

						case 9:
							fMission.mDisplayMessage( kRom2CloakingOut_msg );
							fMission.mPlayVoiceSample( kSndRom2CloakingOut );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 10:
							fMission.mDisplayMessage( kRom2Cloaked_msg );
							fMission.mPlayVoiceSample( kSndRom2Cloaked );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 11:
							fMission.mDisplayMessage( kRom2ApproachPlanet_msg );
							fMission.mPlayVoiceSample( kSndRom2ApproachPlanet );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 12:
							fMission.mDisplayMessage( kRom2Speed_msg );
							fMission.mPlayVoiceSample( kSndRom2Speed );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 13:
							fMission.mDisplayMessage( kRom2Uncloak_msg );
							fMission.mPlayVoiceSample( kSndRom2Uncloak );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 14:
							fMission.mDisplayMessage( kRom2CloakingIn_msg );
							fMission.mPlayVoiceSample( kSndRom2CloakingIn );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 15:
							fMission.mDisplayMessage( kRom2OtherStuff_msg );
							fMission.mPlayVoiceSample( kSndRom2OtherStuff );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 16:
							fMission.mDisplayMessage( kRom2NoCloak_msg );
							fMission.mPlayVoiceSample( kSndRom2NoCloak );
							fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
							break;

						case 17:
							if( MissionEnd == true )
							{
								fMission.mDisplayMessage( kRom2Congratulations_msg );
								fMission.mPlayVoiceSample( kSndRom2Congratulations );
								return kPlayerWins;
							}
							else
							{
								MissionEnd = true;
							}
							break;
					}
					break;

			case kTimerLimitSet_2:	// Loop until voice is done playing...
				if( fMission.mVoiceSampleStillPlaying() != 0 )
				{
					fMission.mSetTimedInterval( kTimerLimitSet_2, 1 );
				}
				else
				{
					TutorialSegment++;
					fMission.mSetTimedInterval( kTimerLimitSet, 1 );
				}
				break;
			}
			break;

		case kNotifyAlertStatusChange:
			if(( Ship->mGetAlertStatus() == kAlertRed ) && ( TutorialSegment >= 5 ))
			{
				TutorialSegment = 6;
				fMission.mSetTimedInterval( kTimerLimitSet, 1 );
			}
			else
			{
				Ship->mSetShipCallbackEvents( kNotifyAlertStatusChange );
			}
			break;

		case kNotifyFlashingButtonPressed:
			{
				TutorialSegment++;
				fMission.mSetTimedInterval( kTimerLimitSet, 1 );
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				fMission.mDisplayMessage( kRom2Died_msg );
				fMission.mPlayVoiceSample( kSndRom2Died );
				return kPlayerLoses;
			}
			break;

		case kNotifyAtDistance:
			if( MissionEnd == true )
			{
				fMission.mDisplayMessage( kRom2Congratulations_msg );
				fMission.mPlayVoiceSample( kSndRom2Congratulations );
				return kPlayerWins;
			}
			else
			{
				MissionEnd = true;
			}
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	return (kAstoundingVictory);
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );

	if( PlayerTeam->mGetShipsInPlayCount() == 0 )
	{
		fMission.mAddToDebriefingList (kRom2Died_msg);
	}
	else
	{
		TutorialPassed = true;
		fMission.mAddToDebriefingList (kRom2Debrief_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	if( TutorialPassed == true )
	{
		return 1;
	}

	return 0;
}
