#include "scriptinterface.h"

#include "MissionText.h"
#include "MissionMaps.h"

enum eMissionAct
{
	kStarting = 0,
	kSabotage,
	kRepairing,
	kRepaired
};

eMissionAct	MissionAct = kStarting;
int32 RepairTotal = 0;
int32 BonusBalancePoints = 0;
int32 DiffLevel = 0;
float PlayerHealth = 0.0;
float EnemyHealth = 0.0;
bool PlayerDisengaged = false;
bool EnemyCaptured = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.

	kEnemyTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAlliedBorder | kUnexplored | kWarzone;
	Specs->fPlayerRace = kAllRaceMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecOneShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 7;
	Specs->fMaxDate = 40;
	Specs->fWait = 99;
	Specs->fEnableSupplyDock = kFalse;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints (150, 100, 50, 25, 0);

	// must set up the races of the teams in the script
	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, info.fEnemyRace);

	// setup team relations and target goals
	fMission.mSetTeamRelations (kEnemyTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0);

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	int32 PlayerBPV = PlayerTeam->mGetCombatBPV();
	DiffLevel = fMission.mGetDifficultyLevel();

	if( DiffLevel == 0 )
	{
		PlayerBPV = PlayerBPV * .75;
	}

	if( DiffLevel == 2 )
	{
		PlayerBPV = PlayerBPV * 1.25;
	}

	EnemyTeam->mCreateShip( PlayerBPV, kClassDestroyer, kClassHeavyCruiser, 0, kStartPosition_Z );
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);

	int32 PBPV = PlayerTeam->mGetCombatBPV();
	int32 EBPV = EnemyTeam->mGetCombatBPV();
	BonusBalancePoints = ( 20 * EBPV / PBPV ) - 20;

	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();

	fMission.mDisplayMessage (kStarting_msg, kEngineerScriptMessage);
	fMission.mSetTimedInterval (kTimerLimitSet, 3);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);

	PlayerHealth = 100.0f - PlayerTeam->mGetTotalDamagePercent();
	EnemyHealth = 100.0f - EnemyTeam->mGetTotalDamagePercent();

	switch (Event)
	{
		case kNotifyAtTimedInterval:
			switch (MissionAct)
			{
				case kStarting:
					fMission.mDisplayMessage (kCaptStart_msg, kMissionScriptMessage);
					fMission.mSetTimedInterval (kTimerLimitSet, 5);
					MissionAct = kSabotage;
					break;

				case kSabotage:
					fMission.mDisplayMessage (kSabotage_msg, kEngineerScriptMessage);
					Ship = PlayerTeam->mGetFirstShip();
					Ship->mDamageSystem (kDamageLeftWarpEngine, 100.0);
					Ship->mDamageSystem (kDamageRightWarpEngine, 100.0);
					Ship->mDamageSystem (kDamageCenterWarpEngine, 100.0);
					fMission.mSetTimedInterval (kTimerTurnsSet, 1);
					MissionAct = kRepairing;
					break;

				case kRepairing:
					RepairTotal = RepairTotal + fMission.mRandomInt32( 1, 6 );
					if (RepairTotal >= 13)
					{
						fMission.mDisplayMessage (kRepaired_msg, kEngineerScriptMessage);
						Ship = PlayerTeam->mGetFirstShip();
						Ship->mDamageSystem (kDamageLeftWarpEngine, 0.0);
						Ship->mDamageSystem (kDamageRightWarpEngine, 0.0);
						Ship->mDamageSystem (kDamageCenterWarpEngine, 0.0);
						MissionAct = kRepaired;
						fMission.mSetTimedInterval( kTimerTurnsSet, fMission.mRandomInt32( 3 + DiffLevel, 6 + DiffLevel ));
					}
					else
					{
						fMission.mDisplayMessage (fMission.mRandomInt32 (kRepairingA_msg, kRepairingE_msg), kEngineerScriptMessage);
						fMission.mSetTimedInterval (kTimerTurnsSet, 1);
					}
					break;

				case kRepaired:
					fMission.mDisplayMessage (kLeaving_msg, kScienceScriptMessage);
					EnemyTeam->mGetFirstShip()->mDisengage(kByMapEdge, kAbsolutePriority);
					break;
			}
			break;

		case kNotifyOnCapture:
			if( Ship->mGetTeam() == kEnemyTeam )
			{
				EnemyCaptured = true;
				fMission.mAddToBonusPrestige( kCaptureBonus );
				fMission.mAddToBonusPrestige( BonusBalancePoints );
				return kEnemyCaptured;
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetOriginalTeam() == kPlayerTeam )
			{
				return kPlayerDestroyed;
			}
			else
			{
				fMission.mAddToBonusPrestige( BonusBalancePoints );
				return kPlayerWins;
			}
			break;

		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				PlayerDisengaged = true;
				return kPlayerWins;
			}
			else
			{
				return kPlayerWins;
			}
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );

	if( PlayerDisengaged == true )
	{
		if( PlayerHealth > EnemyHealth )
		{
			return kVictory;
		}
		else if( PlayerHealth < ( EnemyHealth / 3 ) )
		{
			return kDevastatingDefeat;
		}
		else 
		{
			return kDefeat;
		}
	}

	if(( PlayerHealth > 80.0 ) || ( EnemyCaptured == true ))
	{
		return kAstoundingVictory;
	}

	if( PlayerHealth < 40.0 ) 
	{
		return kDraw;
	}

	// By default
	return kVictory;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList( VictoryLevel + kAstoundingVictory_msg );
	fMission.mAddToDebriefingList( kInvestigating_msg );
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
